class TextBox(TextBoxBase,IComponent,IDisposable,IOleControl,IOleObject,IOleInPlaceObject,IOleInPlaceActiveObject,IOleWindow,IViewObject,IViewObject2,IPersist,IPersistStreamInit,IPersistPropertyBag,IPersistStorage,IQuickActivate,ISupportOleDropSource,IDropTarget,ISynchronizeInvoke,IWin32Window,IArrangedElement,IBindableComponent):
 """
 Represents a Windows text box control.

 

 TextBox()
 """
 def AccessibilityNotifyClients(self,*args):
  """
  AccessibilityNotifyClients(self: Control,accEvent: AccessibleEvents,objectID: int,childID: int)

   Notifies the accessibility client applications of the specified 

    System.Windows.Forms.AccessibleEvents for the specified child control .

  

  

   accEvent: The System.Windows.Forms.AccessibleEvents to notify the accessibility client applications of.

   objectID: The identifier of the System.Windows.Forms.AccessibleObject.

   childID: The child System.Windows.Forms.Control to notify of the accessible event.

  AccessibilityNotifyClients(self: Control,accEvent: AccessibleEvents,childID: int)

   Notifies the accessibility client applications of the specified 

    System.Windows.Forms.AccessibleEvents for the specified child control.

  

  

   accEvent: The System.Windows.Forms.AccessibleEvents to notify the accessibility client applications of.

   childID: The child System.Windows.Forms.Control to notify of the accessible event.
  """
  pass
 def CreateAccessibilityInstance(self,*args):
  """
  CreateAccessibilityInstance(self: Control) -> AccessibleObject

  

   Creates a new accessibility object for the control.

   Returns: A new System.Windows.Forms.AccessibleObject for the control.
  """
  pass
 def CreateControlsInstance(self,*args):
  """
  CreateControlsInstance(self: Control) -> ControlCollection

  

   Creates a new instance of the control collection for the control.

   Returns: A new instance of System.Windows.Forms.Control.ControlCollection assigned to the control.
  """
  pass
 def CreateHandle(self,*args):
  """ CreateHandle(self: TextBoxBase) """
  pass
 def DefWndProc(self,*args):
  """
  DefWndProc(self: Control,m: Message) -> Message

  

   Sends the specified message to the default window procedure.

  

   m: The Windows System.Windows.Forms.Message to process.
  """
  pass
 def DestroyHandle(self,*args):
  """
  DestroyHandle(self: Control)

   Destroys the handle associated with the control.
  """
  pass
 def Dispose(self):
  """
  Dispose(self: TextBox,disposing: bool)

   Releases the unmanaged resources used by the System.Windows.Forms.TextBox and optionally 

    releases the managed resources.

  

  

   disposing: true to release both managed and unmanaged resources; false to release only unmanaged resources.
  """
  pass
 def GetAccessibilityObjectById(self,*args):
  """
  GetAccessibilityObjectById(self: Control,objectId: int) -> AccessibleObject

  

   Retrieves the specified System.Windows.Forms.AccessibleObject.

  

   objectId: An Int32 that identifies the System.Windows.Forms.AccessibleObject to retrieve.

   Returns: An System.Windows.Forms.AccessibleObject.
  """
  pass
 def GetAutoSizeMode(self,*args):
  """
  GetAutoSizeMode(self: Control) -> AutoSizeMode

  

   Retrieves a value indicating how a control will behave when its 

    System.Windows.Forms.Control.AutoSize property is enabled.

  

   Returns: One of the System.Windows.Forms.AutoSizeMode values.
  """
  pass
 def GetScaledBounds(self,*args):
  """
  GetScaledBounds(self: Control,bounds: Rectangle,factor: SizeF,specified: BoundsSpecified) -> Rectangle

  

   Retrieves the bounds within which the control is scaled.

  

   bounds: A System.Drawing.Rectangle that specifies the area for which to retrieve the display bounds.

   factor: The height and width of the control's bounds.

   specified: One of the values of System.Windows.Forms.BoundsSpecified that specifies the bounds of the 

    control to use when defining its size and position.

  

   Returns: A System.Drawing.Rectangle representing the bounds within which the control is scaled.
  """
  pass
 def GetService(self,*args):
  """
  GetService(self: Component,service: Type) -> object

  

   Returns an object that represents a service provided by the System.ComponentModel.Component or 

    by its System.ComponentModel.Container.

  

  

   service: A service provided by the System.ComponentModel.Component.

   Returns: An System.Object that represents a service provided by the System.ComponentModel.Component,or 

    null if the System.ComponentModel.Component does not provide the specified service.
  """
  pass
 def GetStyle(self,*args):
  """
  GetStyle(self: Control,flag: ControlStyles) -> bool

  

   Retrieves the value of the specified control style bit for the control.

  

   flag: The System.Windows.Forms.ControlStyles bit to return the value from.

   Returns: true if the specified control style bit is set to true; otherwise,false.
  """
  pass
 def GetTopLevel(self,*args):
  """
  GetTopLevel(self: Control) -> bool

  

   Determines if the control is a top-level control.

   Returns: true if the control is a top-level control; otherwise,false.
  """
  pass
 def InitLayout(self,*args):
  """
  InitLayout(self: Control)

   Called after the control has been added to another container.
  """
  pass
 def InvokeGotFocus(self,*args):
  """
  InvokeGotFocus(self: Control,toInvoke: Control,e: EventArgs)

   Raises the System.Windows.Forms.Control.GotFocus event for the specified control.

  

   toInvoke: The System.Windows.Forms.Control to assign the event to.

   e: An System.EventArgs that contains the event data.
  """
  pass
 def InvokeLostFocus(self,*args):
  """
  InvokeLostFocus(self: Control,toInvoke: Control,e: EventArgs)

   Raises the System.Windows.Forms.Control.LostFocus event for the specified control.

  

   toInvoke: The System.Windows.Forms.Control to assign the event to.

   e: An System.EventArgs that contains the event data.
  """
  pass
 def InvokeOnClick(self,*args):
  """
  InvokeOnClick(self: Control,toInvoke: Control,e: EventArgs)

   Raises the System.Windows.Forms.Control.Click event for the specified control.

  

   toInvoke: The System.Windows.Forms.Control to assign the System.Windows.Forms.Control.Click event to.

   e: An System.EventArgs that contains the event data.
  """
  pass
 def InvokePaint(self,*args):
  """
  InvokePaint(self: Control,c: Control,e: PaintEventArgs)

   Raises the System.Windows.Forms.Control.Paint event for the specified control.

  

   c: The System.Windows.Forms.Control to assign the System.Windows.Forms.Control.Paint event to.

   e: An System.Windows.Forms.PaintEventArgs that contains the event data.
  """
  pass
 def InvokePaintBackground(self,*args):
  """
  InvokePaintBackground(self: Control,c: Control,e: PaintEventArgs)

   Raises the PaintBackground event for the specified control.

  

   c: The System.Windows.Forms.Control to assign the System.Windows.Forms.Control.Paint event to.

   e: An System.Windows.Forms.PaintEventArgs that contains the event data.
  """
  pass
 def IsInputChar(self,*args):
  """
  IsInputChar(self: Control,charCode: Char) -> bool

  

   Determines if a character is an input character that the control recognizes.

  

   charCode: The character to test.

   Returns: true if the character should be sent directly to the control and not preprocessed; otherwise,

    false.
  """
  pass
 def IsInputKey(self,*args):
  """
  IsInputKey(self: TextBox,keyData: Keys) -> bool

  

   Determines whether the specified key is an input key or a special key that requires 

    preprocessing.

  

  

   keyData: One of the key's values.

   Returns: true if the specified key is an input key; otherwise,false.
  """
  pass
 def MemberwiseClone(self,*args):
  """
  MemberwiseClone(self: MarshalByRefObject,cloneIdentity: bool) -> MarshalByRefObject

  

   Creates a shallow copy of the current System.MarshalByRefObject object.

  

   cloneIdentity: false to delete the current System.MarshalByRefObject object's identity,which will cause the 

    object to be assigned a new identity when it is marshaled across a remoting boundary. A value of 

    false is usually appropriate. true to copy the current System.MarshalByRefObject object's 

    identity to its clone,which will cause remoting client calls to be routed to the remote server 

    object.

  

   Returns: A shallow copy of the current System.MarshalByRefObject object.

  MemberwiseClone(self: object) -> object

  

   Creates a shallow copy of the current System.Object.

   Returns: A shallow copy of the current System.Object.
  """
  pass
 def NotifyInvalidate(self,*args):
  """
  NotifyInvalidate(self: Control,invalidatedArea: Rectangle)

   Raises the System.Windows.Forms.Control.Invalidated event with a specified region of the control 

    to invalidate.

  

  

   invalidatedArea: A System.Drawing.Rectangle representing the area to invalidate.
  """
  pass
 def OnAcceptsTabChanged(self,*args):
  """
  OnAcceptsTabChanged(self: TextBoxBase,e: EventArgs)

   Raises the System.Windows.Forms.TextBoxBase.AcceptsTabChanged event.

  

   e: An System.EventArgs that contains the event data.
  """
  pass
 def OnAutoSizeChanged(self,*args):
  """
  OnAutoSizeChanged(self: Control,e: EventArgs)

   Raises the System.Windows.Forms.Control.AutoSizeChanged event.

  

   e: An System.EventArgs that contains the event data.
  """
  pass
 def OnBackColorChanged(self,*args):
  """
  OnBackColorChanged(self: TextBox,e: EventArgs)

   Raises the System.Windows.Forms.Control.BackColorChanged event.

  

   e: An System.EventArgs that contains the event data.
  """
  pass
 def OnBackgroundImageChanged(self,*args):
  """
  OnBackgroundImageChanged(self: Control,e: EventArgs)

   Raises the System.Windows.Forms.Control.BackgroundImageChanged event.

  

   e: An System.EventArgs that contains the event data.
  """
  pass
 def OnBackgroundImageLayoutChanged(self,*args):
  """
  OnBackgroundImageLayoutChanged(self: Control,e: EventArgs)

   Raises the System.Windows.Forms.Control.BackgroundImageLayoutChanged event.

  

   e: An System.EventArgs that contains the event data.
  """
  pass
 def OnBindingContextChanged(self,*args):
  """
  OnBindingContextChanged(self: Control,e: EventArgs)

   Raises the System.Windows.Forms.Control.BindingContextChanged event.

  

   e: An System.EventArgs that contains the event data.
  """
  pass
 def OnBorderStyleChanged(self,*args):
  """
  OnBorderStyleChanged(self: TextBoxBase,e: EventArgs)

   Raises the System.Windows.Forms.TextBoxBase.BorderStyleChanged event.

  

   e: An System.EventArgs that contains the event data.
  """
  pass
 def OnCausesValidationChanged(self,*args):
  """
  OnCausesValidationChanged(self: Control,e: EventArgs)

   Raises the System.Windows.Forms.Control.CausesValidationChanged event.

  

   e: An System.EventArgs that contains the event data.
  """
  pass
 def OnChangeUICues(self,*args):
  """
  OnChangeUICues(self: Control,e: UICuesEventArgs)

   Raises the System.Windows.Forms.Control.ChangeUICues event.

  

   e: A System.Windows.Forms.UICuesEventArgs that contains the event data.
  """
  pass
 def OnClick(self,*args):
  """
  OnClick(self: Control,e: EventArgs)

   Raises the System.Windows.Forms.Control.Click event.

  

   e: An System.EventArgs that contains the event data.
  """
  pass
 def OnClientSizeChanged(self,*args):
  """
  OnClientSizeChanged(self: Control,e: EventArgs)

   Raises the System.Windows.Forms.Control.ClientSizeChanged event.

  

   e: An System.EventArgs that contains the event data.
  """
  pass
 def OnContextMenuChanged(self,*args):
  """
  OnContextMenuChanged(self: Control,e: EventArgs)

   Raises the System.Windows.Forms.Control.ContextMenuChanged event.

  

   e: An System.EventArgs that contains the event data.
  """
  pass
 def OnContextMenuStripChanged(self,*args):
  """
  OnContextMenuStripChanged(self: Control,e: EventArgs)

   Raises the System.Windows.Forms.Control.ContextMenuStripChanged event.

  

   e: An System.EventArgs that contains the event data.
  """
  pass
 def OnControlAdded(self,*args):
  """
  OnControlAdded(self: Control,e: ControlEventArgs)

   Raises the System.Windows.Forms.Control.ControlAdded event.

  

   e: A System.Windows.Forms.ControlEventArgs that contains the event data.
  """
  pass
 def OnControlRemoved(self,*args):
  """
  OnControlRemoved(self: Control,e: ControlEventArgs)

   Raises the System.Windows.Forms.Control.ControlRemoved event.

  

   e: A System.Windows.Forms.ControlEventArgs that contains the event data.
  """
  pass
 def OnCreateControl(self,*args):
  """
  OnCreateControl(self: Control)

   Raises the System.Windows.Forms.Control.CreateControl method.
  """
  pass
 def OnCursorChanged(self,*args):
  """
  OnCursorChanged(self: Control,e: EventArgs)

   Raises the System.Windows.Forms.Control.CursorChanged event.

  

   e: An System.EventArgs that contains the event data.
  """
  pass
 def OnDockChanged(self,*args):
  """
  OnDockChanged(self: Control,e: EventArgs)

   Raises the System.Windows.Forms.Control.DockChanged event.

  

   e: An System.EventArgs that contains the event data.
  """
  pass
 def OnDoubleClick(self,*args):
  """
  OnDoubleClick(self: Control,e: EventArgs)

   Raises the System.Windows.Forms.Control.DoubleClick event.

  

   e: An System.EventArgs that contains the event data.
  """
  pass
 def OnDpiChangedAfterParent(self,*args):
  """ OnDpiChangedAfterParent(self: Control,e: EventArgs) """
  pass
 def OnDpiChangedBeforeParent(self,*args):
  """ OnDpiChangedBeforeParent(self: Control,e: EventArgs) """
  pass
 def OnDragDrop(self,*args):
  """
  OnDragDrop(self: Control,drgevent: DragEventArgs)

   Raises the System.Windows.Forms.Control.DragDrop event.

  

   drgevent: A System.Windows.Forms.DragEventArgs that contains the event data.
  """
  pass
 def OnDragEnter(self,*args):
  """
  OnDragEnter(self: Control,drgevent: DragEventArgs)

   Raises the System.Windows.Forms.Control.DragEnter event.

  

   drgevent: A System.Windows.Forms.DragEventArgs that contains the event data.
  """
  pass
 def OnDragLeave(self,*args):
  """
  OnDragLeave(self: Control,e: EventArgs)

   Raises the System.Windows.Forms.Control.DragLeave event.

  

   e: An System.EventArgs that contains the event data.
  """
  pass
 def OnDragOver(self,*args):
  """
  OnDragOver(self: Control,drgevent: DragEventArgs)

   Raises the System.Windows.Forms.Control.DragOver event.

  

   drgevent: A System.Windows.Forms.DragEventArgs that contains the event data.
  """
  pass
 def OnEnabledChanged(self,*args):
  """
  OnEnabledChanged(self: Control,e: EventArgs)

   Raises the System.Windows.Forms.Control.EnabledChanged event.

  

   e: An System.EventArgs that contains the event data.
  """
  pass
 def OnEnter(self,*args):
  """
  OnEnter(self: Control,e: EventArgs)

   Raises the System.Windows.Forms.Control.Enter event.

  

   e: An System.EventArgs that contains the event data.
  """
  pass
 def OnFontChanged(self,*args):
  """
  OnFontChanged(self: TextBox,e: EventArgs)

   Raises the System.Windows.Forms.Control.FontChanged event.

  

   e: An System.EventArgs that contains the event data.
  """
  pass
 def OnForeColorChanged(self,*args):
  """
  OnForeColorChanged(self: Control,e: EventArgs)

   Raises the System.Windows.Forms.Control.ForeColorChanged event.

  

   e: An System.EventArgs that contains the event data.
  """
  pass
 def OnGiveFeedback(self,*args):
  """
  OnGiveFeedback(self: Control,gfbevent: GiveFeedbackEventArgs)

   Raises the System.Windows.Forms.Control.GiveFeedback event.

  

   gfbevent: A System.Windows.Forms.GiveFeedbackEventArgs that contains the event data.
  """
  pass
 def OnGotFocus(self,*args):
  """
  OnGotFocus(self: TextBox,e: EventArgs)

   Raises the System.Windows.Forms.Control.GotFocus event.

  

   e: An System.EventArgs that contains the event data.
  """
  pass
 def OnHandleCreated(self,*args):
  """
  OnHandleCreated(self: TextBox,e: EventArgs)

   Raises the System.Windows.Forms.Control.HandleCreated event.

  

   e: The event data.
  """
  pass
 def OnHandleDestroyed(self,*args):
  """
  OnHandleDestroyed(self: TextBox,e: EventArgs)

   Raises the System.Windows.Forms.Control.OnHandleDestroyed(System.EventArgs) event.

  

   e: An System.EventArgs that contains the event data.
  """
  pass
 def OnHelpRequested(self,*args):
  """
  OnHelpRequested(self: Control,hevent: HelpEventArgs)

   Raises the System.Windows.Forms.Control.HelpRequested event.

  

   hevent: A System.Windows.Forms.HelpEventArgs that contains the event data.
  """
  pass
 def OnHideSelectionChanged(self,*args):
  """
  OnHideSelectionChanged(self: TextBoxBase,e: EventArgs)

   Raise the System.Windows.Forms.TextBoxBase.HideSelectionChanged event.

  

   e: An System.EventArgs that contains the event data.
  """
  pass
 def OnImeModeChanged(self,*args):
  """
  OnImeModeChanged(self: Control,e: EventArgs)

   Raises the System.Windows.Forms.Control.ImeModeChanged event.

  

   e: An System.EventArgs that contains the event data.
  """
  pass
 def OnInvalidated(self,*args):
  """
  OnInvalidated(self: Control,e: InvalidateEventArgs)

   Raises the System.Windows.Forms.Control.Invalidated event.

  

   e: An System.Windows.Forms.InvalidateEventArgs that contains the event data.
  """
  pass
 def OnKeyDown(self,*args):
  """
  OnKeyDown(self: Control,e: KeyEventArgs)

   Raises the System.Windows.Forms.Control.KeyDown event.

  

   e: A System.Windows.Forms.KeyEventArgs that contains the event data.
  """
  pass
 def OnKeyPress(self,*args):
  """
  OnKeyPress(self: Control,e: KeyPressEventArgs)

   Raises the System.Windows.Forms.Control.KeyPress event.

  

   e: A System.Windows.Forms.KeyPressEventArgs that contains the event data.
  """
  pass
 def OnKeyUp(self,*args):
  """
  OnKeyUp(self: Control,e: KeyEventArgs)

   Raises the System.Windows.Forms.Control.KeyUp event.

  

   e: A System.Windows.Forms.KeyEventArgs that contains the event data.
  """
  pass
 def OnLayout(self,*args):
  """
  OnLayout(self: Control,levent: LayoutEventArgs)

   Raises the System.Windows.Forms.Control.Layout event.

  

   levent: A System.Windows.Forms.LayoutEventArgs that contains the event data.
  """
  pass
 def OnLeave(self,*args):
  """
  OnLeave(self: Control,e: EventArgs)

   Raises the System.Windows.Forms.Control.Leave event.

  

   e: An System.EventArgs that contains the event data.
  """
  pass
 def OnLocationChanged(self,*args):
  """
  OnLocationChanged(self: Control,e: EventArgs)

   Raises the System.Windows.Forms.Control.LocationChanged event.

  

   e: An System.EventArgs that contains the event data.
  """
  pass
 def OnLostFocus(self,*args):
  """
  OnLostFocus(self: Control,e: EventArgs)

   Raises the System.Windows.Forms.Control.LostFocus event.

  

   e: An System.EventArgs that contains the event data.
  """
  pass
 def OnMarginChanged(self,*args):
  """
  OnMarginChanged(self: Control,e: EventArgs)

   Raises the System.Windows.Forms.Control.MarginChanged event.

  

   e: A System.EventArgs that contains the event data.
  """
  pass
 def OnModifiedChanged(self,*args):
  """
  OnModifiedChanged(self: TextBoxBase,e: EventArgs)

   Raises the System.Windows.Forms.TextBoxBase.ModifiedChanged event.

  

   e: An System.EventArgs that contains the event data.
  """
  pass
 def OnMouseCaptureChanged(self,*args):
  """
  OnMouseCaptureChanged(self: Control,e: EventArgs)

   Raises the System.Windows.Forms.Control.MouseCaptureChanged event.

  

   e: An System.EventArgs that contains the event data.
  """
  pass
 def OnMouseClick(self,*args):
  """
  OnMouseClick(self: Control,e: MouseEventArgs)

   Raises the System.Windows.Forms.Control.MouseClick event.

  

   e: An System.Windows.Forms.MouseEventArgs that contains the event data.
  """
  pass
 def OnMouseDoubleClick(self,*args):
  """
  OnMouseDoubleClick(self: Control,e: MouseEventArgs)

   Raises the System.Windows.Forms.Control.MouseDoubleClick event.

  

   e: An System.Windows.Forms.MouseEventArgs that contains the event data.
  """
  pass
 def OnMouseDown(self,*args):
  """
  OnMouseDown(self: Control,e: MouseEventArgs)

   Raises the System.Windows.Forms.Control.MouseDown event.

  

   e: A System.Windows.Forms.MouseEventArgs that contains the event data.
  """
  pass
 def OnMouseEnter(self,*args):
  """
  OnMouseEnter(self: Control,e: EventArgs)

   Raises the System.Windows.Forms.Control.MouseEnter event.

  

   e: An System.EventArgs that contains the event data.
  """
  pass
 def OnMouseHover(self,*args):
  """
  OnMouseHover(self: Control,e: EventArgs)

   Raises the System.Windows.Forms.Control.MouseHover event.

  

   e: An System.EventArgs that contains the event data.
  """
  pass
 def OnMouseLeave(self,*args):
  """
  OnMouseLeave(self: Control,e: EventArgs)

   Raises the System.Windows.Forms.Control.MouseLeave event.

  

   e: An System.EventArgs that contains the event data.
  """
  pass
 def OnMouseMove(self,*args):
  """
  OnMouseMove(self: Control,e: MouseEventArgs)

   Raises the System.Windows.Forms.Control.MouseMove event.

  

   e: A System.Windows.Forms.MouseEventArgs that contains the event data.
  """
  pass
 def OnMouseUp(self,*args):
  """
  OnMouseUp(self: TextBoxBase,mevent: MouseEventArgs)

   Raises the System.Windows.Forms.Control.MouseUp event.

  

   mevent: The event data.
  """
  pass
 def OnMouseWheel(self,*args):
  """
  OnMouseWheel(self: Control,e: MouseEventArgs)

   Raises the System.Windows.Forms.Control.MouseWheel event.

  

   e: A System.Windows.Forms.MouseEventArgs that contains the event data.
  """
  pass
 def OnMove(self,*args):
  """
  OnMove(self: Control,e: EventArgs)

   Raises the System.Windows.Forms.Control.Move event.

  

   e: An System.EventArgs that contains the event data.
  """
  pass
 def OnMultilineChanged(self,*args):
  """
  OnMultilineChanged(self: TextBoxBase,e: EventArgs)

   Raises the System.Windows.Forms.TextBoxBase.MultilineChanged event.

  

   e: An System.EventArgs that contains the event data.
  """
  pass
 def OnNotifyMessage(self,*args):
  """
  OnNotifyMessage(self: Control,m: Message)

   Notifies the control of Windows messages.

  

   m: A System.Windows.Forms.Message that represents the Windows message.
  """
  pass
 def OnPaddingChanged(self,*args):
  """
  OnPaddingChanged(self: TextBoxBase,e: EventArgs)

   This method is not relevant for this class.

  

   e: An System.EventArgs that contains the event data.
  """
  pass
 def OnPaint(self,*args):
  """
  OnPaint(self: Control,e: PaintEventArgs)

   Raises the System.Windows.Forms.Control.Paint event.

  

   e: A System.Windows.Forms.PaintEventArgs that contains the event data.
  """
  pass
 def OnPaintBackground(self,*args):
  """
  OnPaintBackground(self: Control,pevent: PaintEventArgs)

   Paints the background of the control.

  

   pevent: A System.Windows.Forms.PaintEventArgs that contains information about the control to paint.
  """
  pass
 def OnParentBackColorChanged(self,*args):
  """
  OnParentBackColorChanged(self: Control,e: EventArgs)

   Raises the System.Windows.Forms.Control.BackColorChanged event when the 

    System.Windows.Forms.Control.BackColor property value of the control's container changes.

  

  

   e: An System.EventArgs that contains the event data.
  """
  pass
 def OnParentBackgroundImageChanged(self,*args):
  """
  OnParentBackgroundImageChanged(self: Control,e: EventArgs)

   Raises the System.Windows.Forms.Control.BackgroundImageChanged event when the 

    System.Windows.Forms.Control.BackgroundImage property value of the control's container changes.

  

  

   e: An System.EventArgs that contains the event data.
  """
  pass
 def OnParentBindingContextChanged(self,*args):
  """
  OnParentBindingContextChanged(self: Control,e: EventArgs)

   Raises the System.Windows.Forms.Control.BindingContextChanged event when the 

    System.Windows.Forms.Control.BindingContext property value of the control's container changes.

  

  

   e: An System.EventArgs that contains the event data.
  """
  pass
 def OnParentChanged(self,*args):
  """
  OnParentChanged(self: Control,e: EventArgs)

   Raises the System.Windows.Forms.Control.ParentChanged event.

  

   e: An System.EventArgs that contains the event data.
  """
  pass
 def OnParentCursorChanged(self,*args):
  """
  OnParentCursorChanged(self: Control,e: EventArgs)

   Raises the System.Windows.Forms.Control.CursorChanged event.

  

   e: An System.EventArgs that contains the event data.
  """
  pass
 def OnParentEnabledChanged(self,*args):
  """
  OnParentEnabledChanged(self: Control,e: EventArgs)

   Raises the System.Windows.Forms.Control.EnabledChanged event when the 

    System.Windows.Forms.Control.Enabled property value of the control's container changes.

  

  

   e: An System.EventArgs that contains the event data.
  """
  pass
 def OnParentFontChanged(self,*args):
  """
  OnParentFontChanged(self: Control,e: EventArgs)

   Raises the System.Windows.Forms.Control.FontChanged event when the 

    System.Windows.Forms.Control.Font property value of the control's container changes.

  

  

   e: An System.EventArgs that contains the event data.
  """
  pass
 def OnParentForeColorChanged(self,*args):
  """
  OnParentForeColorChanged(self: Control,e: EventArgs)

   Raises the System.Windows.Forms.Control.ForeColorChanged event when the 

    System.Windows.Forms.Control.ForeColor property value of the control's container changes.

  

  

   e: An System.EventArgs that contains the event data.
  """
  pass
 def OnParentRightToLeftChanged(self,*args):
  """
  OnParentRightToLeftChanged(self: Control,e: EventArgs)

   Raises the System.Windows.Forms.Control.RightToLeftChanged event when the 

    System.Windows.Forms.Control.RightToLeft property value of the control's container changes.

  

  

   e: An System.EventArgs that contains the event data.
  """
  pass
 def OnParentVisibleChanged(self,*args):
  """
  OnParentVisibleChanged(self: Control,e: EventArgs)

   Raises the System.Windows.Forms.Control.VisibleChanged event when the 

    System.Windows.Forms.Control.Visible property value of the control's container changes.

  

  

   e: An System.EventArgs that contains the event data.
  """
  pass
 def OnPreviewKeyDown(self,*args):
  """
  OnPreviewKeyDown(self: Control,e: PreviewKeyDownEventArgs)

   Raises the System.Windows.Forms.Control.PreviewKeyDown event.

  

   e: A System.Windows.Forms.PreviewKeyDownEventArgs that contains the event data.
  """
  pass
 def OnPrint(self,*args):
  """
  OnPrint(self: Control,e: PaintEventArgs)

   Raises the System.Windows.Forms.Control.Paint event.

  

   e: A System.Windows.Forms.PaintEventArgs that contains the event data.
  """
  pass
 def OnQueryContinueDrag(self,*args):
  """
  OnQueryContinueDrag(self: Control,qcdevent: QueryContinueDragEventArgs)

   Raises the System.Windows.Forms.Control.QueryContinueDrag event.

  

   qcdevent: A System.Windows.Forms.QueryContinueDragEventArgs that contains the event data.
  """
  pass
 def OnReadOnlyChanged(self,*args):
  """
  OnReadOnlyChanged(self: TextBoxBase,e: EventArgs)

   Raises the System.Windows.Forms.TextBoxBase.ReadOnlyChanged event.

  

   e: An System.EventArgs that contains the event data.
  """
  pass
 def OnRegionChanged(self,*args):
  """
  OnRegionChanged(self: Control,e: EventArgs)

   Raises the System.Windows.Forms.Control.RegionChanged event.

  

   e: An System.EventArgs that contains the event data.
  """
  pass
 def OnResize(self,*args):
  """
  OnResize(self: Control,e: EventArgs)

   Raises the System.Windows.Forms.Control.Resize event.

  

   e: An System.EventArgs that contains the event data.
  """
  pass
 def OnRightToLeftChanged(self,*args):
  """
  OnRightToLeftChanged(self: Control,e: EventArgs)

   Raises the System.Windows.Forms.Control.RightToLeftChanged event.

  

   e: An System.EventArgs that contains the event data.
  """
  pass
 def OnSizeChanged(self,*args):
  """
  OnSizeChanged(self: Control,e: EventArgs)

   Raises the System.Windows.Forms.Control.SizeChanged event.

  

   e: An System.EventArgs that contains the event data.
  """
  pass
 def OnStyleChanged(self,*args):
  """
  OnStyleChanged(self: Control,e: EventArgs)

   Raises the System.Windows.Forms.Control.StyleChanged event.

  

   e: An System.EventArgs that contains the event data.
  """
  pass
 def OnSystemColorsChanged(self,*args):
  """
  OnSystemColorsChanged(self: Control,e: EventArgs)

   Raises the System.Windows.Forms.Control.SystemColorsChanged event.

  

   e: An System.EventArgs that contains the event data.
  """
  pass
 def OnTabIndexChanged(self,*args):
  """
  OnTabIndexChanged(self: Control,e: EventArgs)

   Raises the System.Windows.Forms.Control.TabIndexChanged event.

  

   e: An System.EventArgs that contains the event data.
  """
  pass
 def OnTabStopChanged(self,*args):
  """
  OnTabStopChanged(self: Control,e: EventArgs)

   Raises the System.Windows.Forms.Control.TabStopChanged event.

  

   e: An System.EventArgs that contains the event data.
  """
  pass
 def OnTextAlignChanged(self,*args):
  """
  OnTextAlignChanged(self: TextBox,e: EventArgs)

   Raises the System.Windows.Forms.TextBox.TextAlignChanged event.

  

   e: An System.EventArgs that contains the event data.
  """
  pass
 def OnTextChanged(self,*args):
  """
  OnTextChanged(self: TextBoxBase,e: EventArgs)

   e: An System.EventArgs that contains the event data.
  """
  pass
 def OnValidated(self,*args):
  """
  OnValidated(self: Control,e: EventArgs)

   Raises the System.Windows.Forms.Control.Validated event.

  

   e: An System.EventArgs that contains the event data.
  """
  pass
 def OnValidating(self,*args):
  """
  OnValidating(self: Control,e: CancelEventArgs)

   Raises the System.Windows.Forms.Control.Validating event.

  

   e: A System.ComponentModel.CancelEventArgs that contains the event data.
  """
  pass
 def OnVisibleChanged(self,*args):
  """
  OnVisibleChanged(self: Control,e: EventArgs)

   Raises the System.Windows.Forms.Control.VisibleChanged event.

  

   e: An System.EventArgs that contains the event data.
  """
  pass
 def Paste(self,text=None):
  """
  Paste(self: TextBox,text: str)

   Sets the selected text to the specified text without clearing the undo buffer.

  

   text: The text to replace.
  """
  pass
 def ProcessCmdKey(self,*args):
  """ ProcessCmdKey(self: TextBox,m: Message,keyData: Keys) -> (bool,Message) """
  pass
 def ProcessDialogChar(self,*args):
  """
  ProcessDialogChar(self: Control,charCode: Char) -> bool

  

   Processes a dialog character.

  

   charCode: The character to process.

   Returns: true if the character was processed by the control; otherwise,false.
  """
  pass
 def ProcessDialogKey(self,*args):
  """
  ProcessDialogKey(self: TextBoxBase,keyData: Keys) -> bool

  

   keyData: One of the System.Windows.Forms.Keys values that represents the key to process.

   Returns: true if the key was processed by the control; otherwise,false.
  """
  pass
 def ProcessKeyEventArgs(self,*args):
  """
  ProcessKeyEventArgs(self: Control,m: Message) -> (bool,Message)

  

   Processes a key message and generates the appropriate control events.

  

   m: A System.Windows.Forms.Message,passed by reference,that represents the window message to 

    process.

  

   Returns: true if the message was processed by the control; otherwise,false.
  """
  pass
 def ProcessKeyMessage(self,*args):
  """
  ProcessKeyMessage(self: Control,m: Message) -> (bool,Message)

  

   Processes a keyboard message.

  

   m: A System.Windows.Forms.Message,passed by reference,that represents the window message to 

    process.

  

   Returns: true if the message was processed by the control; otherwise,false.
  """
  pass
 def ProcessKeyPreview(self,*args):
  """
  ProcessKeyPreview(self: Control,m: Message) -> (bool,Message)

  

   Previews a keyboard message.

  

   m: A System.Windows.Forms.Message,passed by reference,that represents the window message to 

    process.

  

   Returns: true if the message was processed by the control; otherwise,false.
  """
  pass
 def ProcessMnemonic(self,*args):
  """
  ProcessMnemonic(self: Control,charCode: Char) -> bool

  

   Processes a mnemonic character.

  

   charCode: The character to process.

   Returns: true if the character was processed as a mnemonic by the control; otherwise,false.
  """
  pass
 def RaiseDragEvent(self,*args):
  """
  RaiseDragEvent(self: Control,key: object,e: DragEventArgs)

   Raises the appropriate drag event.

  

   key: The event to raise.

   e: A System.Windows.Forms.DragEventArgs that contains the event data.
  """
  pass
 def RaiseKeyEvent(self,*args):
  """
  RaiseKeyEvent(self: Control,key: object,e: KeyEventArgs)

   Raises the appropriate key event.

  

   key: The event to raise.

   e: A System.Windows.Forms.KeyEventArgs that contains the event data.
  """
  pass
 def RaiseMouseEvent(self,*args):
  """
  RaiseMouseEvent(self: Control,key: object,e: MouseEventArgs)

   Raises the appropriate mouse event.

  

   key: The event to raise.

   e: A System.Windows.Forms.MouseEventArgs that contains the event data.
  """
  pass
 def RaisePaintEvent(self,*args):
  """
  RaisePaintEvent(self: Control,key: object,e: PaintEventArgs)

   Raises the appropriate paint event.

  

   key: The event to raise.

   e: A System.Windows.Forms.PaintEventArgs that contains the event data.
  """
  pass
 def RecreateHandle(self,*args):
  """
  RecreateHandle(self: Control)

   Forces the re-creation of the handle for the control.
  """
  pass
 def RescaleConstantsForDpi(self,*args):
  """ RescaleConstantsForDpi(self: Control,deviceDpiOld: int,deviceDpiNew: int) """
  pass
 def ResetMouseEventArgs(self,*args):
  """
  ResetMouseEventArgs(self: Control)

   Resets the control to handle the System.Windows.Forms.Control.MouseLeave event.
  """
  pass
 def RtlTranslateAlignment(self,*args):
  """
  RtlTranslateAlignment(self: Control,align: ContentAlignment) -> ContentAlignment

  

   Converts the specified System.Drawing.ContentAlignment to the appropriate 

    System.Drawing.ContentAlignment to support right-to-left text.

  

  

   align: One of the System.Drawing.ContentAlignment values.

   Returns: One of the System.Drawing.ContentAlignment values.

  RtlTranslateAlignment(self: Control,align: LeftRightAlignment) -> LeftRightAlignment

  

   Converts the specified System.Windows.Forms.LeftRightAlignment to the appropriate 

    System.Windows.Forms.LeftRightAlignment to support right-to-left text.

  

  

   align: One of the System.Windows.Forms.LeftRightAlignment values.

   Returns: One of the System.Windows.Forms.LeftRightAlignment values.

  RtlTranslateAlignment(self: Control,align: HorizontalAlignment) -> HorizontalAlignment

  

   Converts the specified System.Windows.Forms.HorizontalAlignment to the appropriate 

    System.Windows.Forms.HorizontalAlignment to support right-to-left text.

  

  

   align: One of the System.Windows.Forms.HorizontalAlignment values.

   Returns: One of the System.Windows.Forms.HorizontalAlignment values.
  """
  pass
 def RtlTranslateContent(self,*args):
  """
  RtlTranslateContent(self: Control,align: ContentAlignment) -> ContentAlignment

  

   Converts the specified System.Drawing.ContentAlignment to the appropriate 

    System.Drawing.ContentAlignment to support right-to-left text.

  

  

   align: One of the System.Drawing.ContentAlignment values.

   Returns: One of the System.Drawing.ContentAlignment values.
  """
  pass
 def RtlTranslateHorizontal(self,*args):
  """
  RtlTranslateHorizontal(self: Control,align: HorizontalAlignment) -> HorizontalAlignment

  

   Converts the specified System.Windows.Forms.HorizontalAlignment to the appropriate 

    System.Windows.Forms.HorizontalAlignment to support right-to-left text.

  

  

   align: One of the System.Windows.Forms.HorizontalAlignment values.

   Returns: One of the System.Windows.Forms.HorizontalAlignment values.
  """
  pass
 def RtlTranslateLeftRight(self,*args):
  """
  RtlTranslateLeftRight(self: Control,align: LeftRightAlignment) -> LeftRightAlignment

  

   Converts the specified System.Windows.Forms.LeftRightAlignment to the appropriate 

    System.Windows.Forms.LeftRightAlignment to support right-to-left text.

  

  

   align: One of the System.Windows.Forms.LeftRightAlignment values.

   Returns: One of the System.Windows.Forms.LeftRightAlignment values.
  """
  pass
 def ScaleControl(self,*args):
  """
  ScaleControl(self: Control,factor: SizeF,specified: BoundsSpecified)

   Scales a control's location,size,padding and margin.

  

   factor: The factor by which the height and width of the control will be scaled.

   specified: A System.Windows.Forms.BoundsSpecified value that specifies the bounds of the control to use 

    when defining its size and position.
  """
  pass
 def ScaleCore(self,*args):
  """
  ScaleCore(self: Control,dx: Single,dy: Single)

   This method is not relevant for this class.

  

   dx: The horizontal scaling factor.

   dy: The vertical scaling factor.
  """
  pass
 def Select(self,start=None,length=None):
  """
  Select(self: Control,directed: bool,forward: bool)

   Activates a child control. Optionally specifies the direction in the tab order to select the 

    control from.

  

  

   directed: true to specify the direction of the control to select; otherwise,false.

   forward: true to move forward in the tab order; false to move backward in the tab order.
  """
  pass
 def SetAutoSizeMode(self,*args):
  """
  SetAutoSizeMode(self: Control,mode: AutoSizeMode)

   Sets a value indicating how a control will behave when its System.Windows.Forms.Control.AutoSize 

    property is enabled.

  

  

   mode: One of the System.Windows.Forms.AutoSizeMode values.
  """
  pass
 def SetBoundsCore(self,*args):
  """
  SetBoundsCore(self: TextBoxBase,x: int,y: int,width: int,height: int,specified: BoundsSpecified)

   Sets the specified bounds of the System.Windows.Forms.TextBoxBase control.

  

   x: The new System.Windows.Forms.Control.Left property value of the control.

   y: The new System.Windows.Forms.Control.Top property value of the control.

   width: The new System.Windows.Forms.Control.Width property value of the control.

   height: Not used.

   specified: A bitwise combination of the System.Windows.Forms.BoundsSpecified values.
  """
  pass
 def SetClientSizeCore(self,*args):
  """
  SetClientSizeCore(self: Control,x: int,y: int)

   Sets the size of the client area of the control.

  

   x: The client area width,in pixels.

   y: The client area height,in pixels.
  """
  pass
 def SetStyle(self,*args):
  """
  SetStyle(self: Control,flag: ControlStyles,value: bool)

   Sets a specified System.Windows.Forms.ControlStyles flag to either true or false.

  

   flag: The System.Windows.Forms.ControlStyles bit to set.

   value: true to apply the specified style to the control; otherwise,false.
  """
  pass
 def SetTopLevel(self,*args):
  """
  SetTopLevel(self: Control,value: bool)

   Sets the control as the top-level control.

  

   value: true to set the control as the top-level control; otherwise,false.
  """
  pass
 def SetVisibleCore(self,*args):
  """
  SetVisibleCore(self: Control,value: bool)

   Sets the control to the specified visible state.

  

   value: true to make the control visible; otherwise,false.
  """
  pass
 def SizeFromClientSize(self,*args):
  """
  SizeFromClientSize(self: Control,clientSize: Size) -> Size

  

   Determines the size of the entire control from the height and width of its client area.

  

   clientSize: A System.Drawing.Size value representing the height and width of the control's client area.

   Returns: A System.Drawing.Size value representing the height and width of the entire control.
  """
  pass
 def UpdateBounds(self,*args):
  """
  UpdateBounds(self: Control,x: int,y: int,width: int,height: int,clientWidth: int,clientHeight: int)

   Updates the bounds of the control with the specified size,location,and client size.

  

   x: The System.Drawing.Point.X coordinate of the control.

   y: The System.Drawing.Point.Y coordinate of the control.

   width: The System.Drawing.Size.Width of the control.

   height: The System.Drawing.Size.Height of the control.

   clientWidth: The client System.Drawing.Size.Width of the control.

   clientHeight: The client System.Drawing.Size.Height of the control.

  UpdateBounds(self: Control,x: int,y: int,width: int,height: int)

   Updates the bounds of the control with the specified size and location.

  

   x: The System.Drawing.Point.X coordinate of the control.

   y: The System.Drawing.Point.Y coordinate of the control.

   width: The System.Drawing.Size.Width of the control.

   height: The System.Drawing.Size.Height of the control.

  UpdateBounds(self: Control)

   Updates the bounds of the control with the current size and location.
  """
  pass
 def UpdateStyles(self,*args):
  """
  UpdateStyles(self: Control)

   Forces the assigned styles to be reapplied to the control.
  """
  pass
 def UpdateZOrder(self,*args):
  """
  UpdateZOrder(self: Control)

   Updates the control in its parent's z-order.
  """
  pass
 def WndProc(self,*args):
  """
  WndProc(self: TextBox,m: Message) -> Message

  

   Processes Windows messages.

  

   m: A Windows Message object.
  """
  pass
 def __enter__(self,*args):
  """
  __enter__(self: IDisposable) -> object

  

   Provides the implementation of __enter__ for objects which implement IDisposable.
  """
  pass
 def __exit__(self,*args):
  """
  __exit__(self: IDisposable,exc_type: object,exc_value: object,exc_back: object)

   Provides the implementation of __exit__ for objects which implement IDisposable.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __str__(self,*args):
  pass
 AcceptsReturn=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value indicating whether pressing ENTER in a multiline System.Windows.Forms.TextBox control creates a new line of text in the control or activates the default button for the form.



Get: AcceptsReturn(self: TextBox) -> bool



Set: AcceptsReturn(self: TextBox)=value

"""

 AutoCompleteCustomSource=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a custom System.Collections.Specialized.StringCollection to use when the System.Windows.Forms.TextBox.AutoCompleteSource property is set to CustomSource.



Get: AutoCompleteCustomSource(self: TextBox) -> AutoCompleteStringCollection



Set: AutoCompleteCustomSource(self: TextBox)=value

"""

 AutoCompleteMode=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets an option that controls how automatic completion works for the System.Windows.Forms.TextBox.



Get: AutoCompleteMode(self: TextBox) -> AutoCompleteMode



Set: AutoCompleteMode(self: TextBox)=value

"""

 AutoCompleteSource=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value specifying the source of complete strings used for automatic completion.



Get: AutoCompleteSource(self: TextBox) -> AutoCompleteSource



Set: AutoCompleteSource(self: TextBox)=value

"""

 CanEnableIme=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value indicating whether the System.Windows.Forms.Control.ImeMode property can be set to an active value,to enable IME support.



"""

 CanRaiseEvents=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Determines if events can be raised on the control.



"""

 CharacterCasing=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets whether the System.Windows.Forms.TextBox control modifies the case of characters as they are typed.



Get: CharacterCasing(self: TextBox) -> CharacterCasing



Set: CharacterCasing(self: TextBox)=value

"""

 CreateParams=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the required creation parameters when the control handle is created.



"""

 DefaultCursor=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the default cursor for the control.



"""

 DefaultImeMode=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the default Input Method Editor (IME) mode supported by the control.



"""

 DefaultMargin=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the space,in pixels,that is specified by default between controls.



"""

 DefaultMaximumSize=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the length and height,in pixels,that is specified as the default maximum size of a control.



"""

 DefaultMinimumSize=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the length and height,in pixels,that is specified as the default minimum size of a control.



"""

 DefaultPadding=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the internal spacing,in pixels,of the contents of a control.



"""

 DefaultSize=property(lambda self: object(),lambda self,v: None,lambda self: None)

 DesignMode=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether the System.ComponentModel.Component is currently in design mode.



"""

 DoubleBuffered=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value indicating whether control drawing is done in a buffer before the control is displayed. This property is not relevant for this class.



"""

 Events=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the list of event handlers that are attached to this System.ComponentModel.Component.



"""

 FontHeight=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the height of the font of the control.



"""

 ImeModeBase=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the Input Method Editor (IME) mode of a control.



"""

 Multiline=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value indicating whether this is a multiline System.Windows.Forms.TextBox control.



Get: Multiline(self: TextBox) -> bool



Set: Multiline(self: TextBox)=value

"""

 PasswordChar=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the character used to mask characters of a password in a single-line System.Windows.Forms.TextBox control.



Get: PasswordChar(self: TextBox) -> Char



Set: PasswordChar(self: TextBox)=value

"""

 RenderRightToLeft=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """This property is now obsolete.



"""

 ResizeRedraw=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value indicating whether the control redraws itself when resized.



"""

 ScaleChildren=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that determines the scaling of child controls.



"""

 ScrollBars=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets which scroll bars should appear in a multiline System.Windows.Forms.TextBox control.



Get: ScrollBars(self: TextBox) -> ScrollBars



Set: ScrollBars(self: TextBox)=value

"""

 ShowFocusCues=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value indicating whether the control should display focus rectangles.



"""

 ShowKeyboardCues=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value indicating whether the user interface is in the appropriate state to show or hide keyboard accelerators.



"""

 Text=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the current text in the System.Windows.Forms.TextBox.



Get: Text(self: TextBox) -> str



Set: Text(self: TextBox)=value

"""

 TextAlign=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets how text is aligned in a System.Windows.Forms.TextBox control.



Get: TextAlign(self: TextBox) -> HorizontalAlignment



Set: TextAlign(self: TextBox)=value

"""

 UseSystemPasswordChar=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value indicating whether the text in the System.Windows.Forms.TextBox control should appear as the default password character.



Get: UseSystemPasswordChar(self: TextBox) -> bool



Set: UseSystemPasswordChar(self: TextBox)=value

"""


 TextAlignChanged=None




class _PropertyInfo:
 """ Exposes the public members of the System.Reflection.PropertyInfo class to unmanaged code. """
 def Equals(self,other):
  """
  Equals(self: _PropertyInfo,other: object) -> bool

  

   Provides COM objects with version-independent access to the System.Object.Equals(System.Object) 

    method.

  

  

   other: The System.Object to compare with the current System.Object.

   Returns: true if the specified System.Object is equal to the current System.Object; otherwise,false.
  """
  pass
 def GetAccessors(self,nonPublic=None):
  """
  GetAccessors(self: _PropertyInfo) -> Array[MethodInfo]

  

   Provides COM objects with version-independent access to the 

    System.Reflection.PropertyInfo.GetAccessors method.

  

   Returns: An array of System.Reflection.MethodInfo objects that reflect the public get,set,and other 

    accessors of the property reflected by the current instance,if accessors are found; otherwise,

    this method returns an array with zero (0) elements.

  

  GetAccessors(self: _PropertyInfo,nonPublic: bool) -> Array[MethodInfo]

  

   Provides COM objects with version-independent access to the 

    System.Reflection.PropertyInfo.GetAccessors(System.Boolean) method.

  

  

   nonPublic: true to include non-public methods in the returned MethodInfo array; otherwise,false.

   Returns: An array of System.Reflection.MethodInfo objects whose elements reflect the get,set,and other 

    accessors of the property reflected by the current instance. If the nonPublic parameter is true,

    this array contains public and non-public get,set,and other accessors. If nonPublic is false,

    this array contains only public get,set,and other accessors. If no accessors with the 

    specified visibility are found,this method returns an array with zero (0) elements.
  """
  pass
 def GetCustomAttributes(self,*__args):
  """
  GetCustomAttributes(self: _PropertyInfo,inherit: bool) -> Array[object]

  

   Provides COM objects with version-independent access to the 

    System.Reflection.MemberInfo.GetCustomAttributes(System.Boolean) method.

  

  

   inherit: true to search this member's inheritance chain to find the attributes; otherwise false.

   Returns: An array that contains all the custom attributes,or an array with zero elements if no 

    attributes are defined.

  

  GetCustomAttributes(self: _PropertyInfo,attributeType: Type,inherit: bool) -> Array[object]

  

   Provides COM objects with version-independent access to the 

    System.Reflection.MemberInfo.GetCustomAttributes(System.Type,System.Boolean) method.

  

  

   attributeType: The type of attribute to search for. Only attributes that are assignable to this type are 

    returned.

  

   inherit: true to search this member's inheritance chain to find the attributes; otherwise false.

   Returns: An array of custom attributes applied to this member,or an array with zero (0) elements if no 

    attributes have been applied.
  """
  pass
 def GetGetMethod(self,nonPublic=None):
  """
  GetGetMethod(self: _PropertyInfo) -> MethodInfo

  

   Provides COM objects with version-independent access to the 

    System.Reflection.PropertyInfo.GetGetMethod method.

  

   Returns: A System.Reflection.MethodInfo object representing the public get accessor for this property,or 

    null if the get accessor is non-public or does not exist.

  

  GetGetMethod(self: _PropertyInfo,nonPublic: bool) -> MethodInfo

  

   Provides COM objects with version-independent access to the 

    System.Reflection.PropertyInfo.GetGetMethod(System.Boolean) method.

  

  

   nonPublic: true to return a non-public get accessor; otherwise,false.

   Returns: A System.Reflection.MethodInfo object representing the get accessor for this property,if the 

    nonPublic parameter is true. Or null if nonPublic is false and the get accessor is non-public,

    or if nonPublic is true but no get accessors exist.
  """
  pass
 def GetHashCode(self):
  """
  GetHashCode(self: _PropertyInfo) -> int

  

   Provides COM objects with version-independent access to the System.Object.GetHashCode method.

   Returns: The hash code for the current instance.
  """
  pass
 def GetIDsOfNames(self,riid,rgszNames,cNames,lcid,rgDispId):
  """
  GetIDsOfNames(self: _PropertyInfo,riid: Guid,rgszNames: IntPtr,cNames: UInt32,lcid: UInt32,rgDispId: IntPtr) -> Guid

  

   Maps a set of names to a corresponding set of dispatch identifiers.

  

   riid: Reserved for future use. Must be IID_NULL.

   rgszNames: An array of names to be mapped.

   cNames: The count of the names to be mapped.

   lcid: The locale context in which to interpret the names.

   rgDispId: An array allocated by the caller that receives the identifiers corresponding to the names.
  """
  pass
 def GetIndexParameters(self):
  """
  GetIndexParameters(self: _PropertyInfo) -> Array[ParameterInfo]

  

   Provides COM objects with version-independent access to the 

    System.Reflection.PropertyInfo.GetIndexParameters method.

  

   Returns: An array of type System.Reflection.ParameterInfo containing the parameters for the indexes.
  """
  pass
 def GetSetMethod(self,nonPublic=None):
  """
  GetSetMethod(self: _PropertyInfo) -> MethodInfo

  

   Provides COM objects with version-independent access to the 

    System.Reflection.PropertyInfo.GetSetMethod method.

  

   Returns: The System.Reflection.MethodInfo object representing the Set method for this property if the set 

    accessor is public,or null if the set accessor is not public.

  

  GetSetMethod(self: _PropertyInfo,nonPublic: bool) -> MethodInfo

  

   Provides COM objects with version-independent access to the 

    System.Reflection.PropertyInfo.GetSetMethod(System.Boolean) method.

  

  

   nonPublic: true to return a non-public accessor; otherwise,false.

   Returns: One of the values in the following table.Value Meaning A System.Reflection.MethodInfo object 

    representing the Set method for this property. The set accessor is public.-or- The nonPublic 

    parameter is true and the set accessor is non-public. nullThe nonPublic parameter is true,but 

    the property is read-only.-or- The nonPublic parameter is false and the set accessor is 

    non-public.-or- There is no set accessor.
  """
  pass
 def GetType(self):
  """
  GetType(self: _PropertyInfo) -> Type

  

   Provides COM objects with version-independent access to the System.Object.GetType method.

   Returns: A System.Type object.
  """
  pass
 def GetTypeInfo(self,iTInfo,lcid,ppTInfo):
  """
  GetTypeInfo(self: _PropertyInfo,iTInfo: UInt32,lcid: UInt32,ppTInfo: IntPtr)

   Retrieves the type information for an object,which can be used to get the type information for 

    an interface.

  

  

   iTInfo: The type information to return.

   lcid: The locale identifier for the type information.

   ppTInfo: A pointer to the requested type information object.
  """
  pass
 def GetTypeInfoCount(self,pcTInfo):
  """
  GetTypeInfoCount(self: _PropertyInfo) -> UInt32

  

   Retrieves the number of type information interfaces that an object provides (either 0 or 1).
  """
  pass
 def GetValue(self,obj,*__args):
  """
  GetValue(self: _PropertyInfo,obj: object,invokeAttr: BindingFlags,binder: Binder,index: Array[object],culture: CultureInfo) -> object

  

   Provides COM objects with version-independent access to the 

    System.Reflection.PropertyInfo.GetValue(System.Object,System.Reflection.BindingFlags,System.Refle

    ction.Binder,System.Object[],System.Globalization.CultureInfo) method.

  

  

   obj: The object whose property value will be returned.

   invokeAttr: The invocation attribute. This must be a bit flag from BindingFlags: InvokeMethod,

    CreateInstance,Static,GetField,SetField,GetProperty,or SetProperty. A suitable invocation 

    attribute must be specified. If a static member will be invoked,the Static flag of BindingFlags 

    must be set.

  

   binder: An object that enables the binding,coercion of argument types,invocation of members,and 

    retrieval of MemberInfo objects through reflection. If binder is null,the default binder is 

    used.

  

   index: Optional index values for indexed properties. This value should be null for non-indexed 

    properties.

  

   culture: The CultureInfo object that represents the culture for which the resource will be localized. 

    Note that if the resource is not localized for this culture,the CultureInfo.Parent method will 

    be called successively in search of a match. If this value is null,the CultureInfo is obtained 

    from the CultureInfo.CurrentUICulture property.

  

   Returns: The property value for the obj parameter.

  GetValue(self: _PropertyInfo,obj: object,index: Array[object]) -> object

  

   Provides COM objects with version-independent access to the 

    System.Reflection.PropertyInfo.GetValue(System.Object,System.Object[]) method.

  

  

   obj: The object whose property value will be returned.

   index: Optional index values for indexed properties. This value should be null for non-indexed 

    properties.

  

   Returns: The property value for the obj parameter.
  """
  pass
 def Invoke(self,dispIdMember,riid,lcid,wFlags,pDispParams,pVarResult,pExcepInfo,puArgErr):
  """
  Invoke(self: _PropertyInfo,dispIdMember: UInt32,riid: Guid,lcid: UInt32,wFlags: Int16,pDispParams: IntPtr,pVarResult: IntPtr,pExcepInfo: IntPtr,puArgErr: IntPtr) -> Guid

  

   Provides access to properties and methods exposed by an object.

  

   dispIdMember: An identifier of a member.

   riid: Reserved for future use. Must be IID_NULL.

   lcid: The locale context in which to interpret arguments.

   wFlags: Flags describing the context of the call.

   pDispParams: A pointer to a structure containing an array of arguments,an array of argument DISPIDs for 

    named arguments,and counts for the number of elements in the arrays.

  

   pVarResult: A pointer to the location where the result will be stored.

   pExcepInfo: A pointer to a structure that contains exception information.

   puArgErr: The index of the first argument that has an error.
  """
  pass
 def IsDefined(self,attributeType,inherit):
  """
  IsDefined(self: _PropertyInfo,attributeType: Type,inherit: bool) -> bool

  

   Provides COM objects with version-independent access to the 

    System.Reflection.MemberInfo.IsDefined(System.Type,System.Boolean) method.

  

  

   attributeType: The System.Type object to which the custom attributes are applied.

   inherit: true to search this member's inheritance chain to find the attributes; otherwise false.

   Returns: true if one or more instances of the attributeType parameter are applied to this member; 

    otherwise,false.
  """
  pass
 def SetValue(self,obj,value,*__args):
  """
  SetValue(self: _PropertyInfo,obj: object,value: object,invokeAttr: BindingFlags,binder: Binder,index: Array[object],culture: CultureInfo)

   Provides COM objects with version-independent access to the 

    System.Reflection.FieldInfo.SetValue(System.Object,System.Object,System.Reflection.BindingFlags,S

    ystem.Reflection.Binder,System.Globalization.CultureInfo) method.

  

  

   obj: The object whose property value will be returned.

   value: The new value for this property.

   invokeAttr: The invocation attribute. This must be a bit flag from System.Reflection.BindingFlags: 

    InvokeMethod,CreateInstance,Static,GetField,SetField,GetProperty,or SetProperty. A 

    suitable invocation attribute must be specified. If a static member will be invoked,the Static 

    flag of BindingFlags must be set.

  

   binder: An object that enables the binding,coercion of argument types,invocation of members,and 

    retrieval of System.Reflection.MemberInfo objects through reflection. If binder is null,the 

    default binder is used.

  

   index: Optional index values for indexed properties. This value should be null for non-indexed 

    properties.

  

   culture: The System.Globalization.CultureInfo object that represents the culture for which the resource 

    will be localized. Note that if the resource is not localized for this culture,the 

    CultureInfo.Parent method will be called successively in search of a match. If this value is 

    null,the CultureInfo is obtained from the CultureInfo.CurrentUICulture property.

  

  SetValue(self: _PropertyInfo,obj: object,value: object,index: Array[object])

   Provides COM objects with version-independent access to the 

    System.Reflection.PropertyInfo.SetValue(System.Object,System.Object,System.Object[]) method.

  

  

   obj: The object whose property value will be set.

   value: The new value for this property.

   index: Optional index values for indexed properties. This value should be null for non-indexed 

    properties.
  """
  pass
 def ToString(self):
  """
  ToString(self: _PropertyInfo) -> str

  

   Provides COM objects with version-independent access to the System.Object.ToString method.

   Returns: A string that represents the current System.Object.
  """
  pass
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==y """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __str__(self,*args):
  pass
 Attributes=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Provides COM objects with version-independent access to the System.Reflection.PropertyInfo.Attributes property.



Get: Attributes(self: _PropertyInfo) -> PropertyAttributes



"""

 CanRead=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Provides COM objects with version-independent access to the System.Reflection.PropertyInfo.CanRead property.



Get: CanRead(self: _PropertyInfo) -> bool



"""

 CanWrite=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Provides COM objects with version-independent access to the System.Reflection.PropertyInfo.CanWrite property.



Get: CanWrite(self: _PropertyInfo) -> bool



"""

 DeclaringType=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Provides COM objects with version-independent access to the System.Reflection.MemberInfo.DeclaringType property.



Get: DeclaringType(self: _PropertyInfo) -> Type



"""

 IsSpecialName=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Provides COM objects with version-independent access to the System.Reflection.PropertyInfo.IsSpecialName property.



Get: IsSpecialName(self: _PropertyInfo) -> bool



"""

 MemberType=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Provides COM objects with version-independent access to the System.Reflection.PropertyInfo.MemberType property.



Get: MemberType(self: _PropertyInfo) -> MemberTypes



"""

 Name=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Provides COM objects with version-independent access to the System.Reflection.MemberInfo.Name property.



Get: Name(self: _PropertyInfo) -> str



"""

 PropertyType=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Provides COM objects with version-independent access to the System.Reflection.PropertyInfo.PropertyType property.



Get: PropertyType(self: _PropertyInfo) -> Type



"""

 ReflectedType=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Provides COM objects with version-independent access to the System.Reflection.MemberInfo.ReflectedType property.



Get: ReflectedType(self: _PropertyInfo) -> Type



"""





class CursorConverter(TypeConverter):
 """
 Provides a type converter to convert System.Windows.Forms.Cursor objects to and from various other representations.

 

 CursorConverter()
 """
 def CanConvertFrom(self,*__args):
  """
  CanConvertFrom(self: CursorConverter,context: ITypeDescriptorContext,sourceType: Type) -> bool

  

   Determines if this converter can convert an object in the given source type to the native type 

    of the converter.

  

  

   context: A formatter context. This object can be used to extract additional information about the 

    environment this converter is being invoked from. This may be null,so you should always check. 

    Also,properties on the context object may also return null.

  

   sourceType: The type you wish to convert from.

   Returns: true if this object can perform the conversion.
  """
  pass
 def CanConvertTo(self,*__args):
  """
  CanConvertTo(self: CursorConverter,context: ITypeDescriptorContext,destinationType: Type) -> bool

  

   Gets a value indicating whether this converter can convert an object to the given destination 

    type using the context.

  

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context.

   destinationType: A System.Type that represents the type you wish to convert to.

   Returns: true if this converter can perform the conversion; otherwise,false.
  """
  pass
 def ConvertFrom(self,*__args):
  """
  ConvertFrom(self: CursorConverter,context: ITypeDescriptorContext,culture: CultureInfo,value: object) -> object

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context.

   culture: The System.Globalization.CultureInfo to use as the current culture.

   value: The System.Object to convert.

   Returns: An System.Object that represents the converted value.
  """
  pass
 def ConvertTo(self,*__args):
  """
  ConvertTo(self: CursorConverter,context: ITypeDescriptorContext,culture: CultureInfo,value: object,destinationType: Type) -> object

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context.

   culture: A System.Globalization.CultureInfo. If null is passed,the current culture is assumed.

   value: The System.Object to convert.

   destinationType: The System.Type to convert the value parameter to.

   Returns: An System.Object that represents the converted value.
  """
  pass
 def GetStandardValues(self,context=None):
  """
  GetStandardValues(self: CursorConverter,context: ITypeDescriptorContext) -> StandardValuesCollection

  

   Retrieves a collection containing a set of standard values for the data type this validator is 

    designed for. This will return null if the data type does not support a standard set of values.

  

  

   context: A formatter context. This object can be used to extract additional information about the 

    environment this converter is being invoked from. This may be null,so you should always check. 

    Also,properties on the context object may also return null.

  

   Returns: A collection containing a standard set of valid values,or null. The default implementation 

    always returns null.
  """
  pass
 def GetStandardValuesSupported(self,context=None):
  """
  GetStandardValuesSupported(self: CursorConverter,context: ITypeDescriptorContext) -> bool

  

   Determines if this object supports a standard set of values that can be picked from a list.

  

   context: A type descriptor through which additional context may be provided.

   Returns: Returns true if GetStandardValues should be called to find a common set of values the object 

    supports.
  """
  pass



class GridTablesFactory(object):
 """ Provides the System.Windows.Forms.GridTablesFactory.CreateGridTables(System.Windows.Forms.DataGridTableStyle,System.Object,System.String,System.Windows.Forms.BindingContext) method. """
 @staticmethod
 def CreateGridTables(gridTable,dataSource,dataMember,bindingManager):
  """
  CreateGridTables(gridTable: DataGridTableStyle,dataSource: object,dataMember: str,bindingManager: BindingContext) -> Array[DataGridTableStyle]

  

   Returns the specified System.Windows.Forms.DataGridColumnStyle.DataGridTableStyle in a 

    one-element array.

  

  

   gridTable: A System.Windows.Forms.DataGridTableStyle.

   dataSource: An System.Object.

   dataMember: A System.String.

   bindingManager: A System.Windows.Forms.BindingContext.

   Returns: An array of System.Windows.Forms.DataGridTableStyle objects.
  """
  pass



class ArgumentException(SystemException,ISerializable,_Exception):
 """
 The exception that is thrown when one of the arguments provided to a method is not valid.

 

 ArgumentException()

 ArgumentException(message: str)

 ArgumentException(message: str,innerException: Exception)

 ArgumentException(message: str,paramName: str,innerException: Exception)

 ArgumentException(message: str,paramName: str)
 """
 def add_SerializeObjectState(self,*args):
  """ add_SerializeObjectState(self: Exception,value: EventHandler[SafeSerializationEventArgs]) """
  pass
 def GetObjectData(self,info,context):
  """
  GetObjectData(self: ArgumentException,info: SerializationInfo,context: StreamingContext)

   Sets the System.Runtime.Serialization.SerializationInfo object with the parameter name and 

    additional exception information.

  

  

   info: The object that holds the serialized object data.

   context: The contextual information about the source or destination.
  """
  pass
 def remove_SerializeObjectState(self,*args):
  """ remove_SerializeObjectState(self: Exception,value: EventHandler[SafeSerializationEventArgs]) """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,message=None,*__args):
  """
  __new__(cls: type)

  __new__(cls: type,message: str)

  __new__(cls: type,message: str,innerException: Exception)

  __new__(cls: type,message: str,paramName: str,innerException: Exception)

  __new__(cls: type,message: str,paramName: str)

  __new__(cls: type,info: SerializationInfo,context: StreamingContext)
  """
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 Message=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the error message and the parameter name,or only the error message if no parameter name is set.



Get: Message(self: ArgumentException) -> str



"""

 ParamName=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the name of the parameter that causes this exception.



Get: ParamName(self: ArgumentException) -> str



"""





class FileSystemInfo(MarshalByRefObject,ISerializable):
 """ Provides the base class for both System.IO.FileInfo and System.IO.DirectoryInfo objects. """
 def Delete(self):
  """
  Delete(self: FileSystemInfo)

   Deletes a file or directory.
  """
  pass
 def GetObjectData(self,info,context):
  """
  GetObjectData(self: FileSystemInfo,info: SerializationInfo,context: StreamingContext)

   Sets the System.Runtime.Serialization.SerializationInfo object with the file name and additional 

    exception information.

  

  

   info: The System.Runtime.Serialization.SerializationInfo that holds the serialized object data about 

    the exception being thrown.

  

   context: The System.Runtime.Serialization.StreamingContext that contains contextual information about the 

    source or destination.
  """
  pass
 def MemberwiseClone(self,*args):
  """
  MemberwiseClone(self: MarshalByRefObject,cloneIdentity: bool) -> MarshalByRefObject

  

   Creates a shallow copy of the current System.MarshalByRefObject object.

  

   cloneIdentity: false to delete the current System.MarshalByRefObject object's identity,which will cause the 

    object to be assigned a new identity when it is marshaled across a remoting boundary. A value of 

    false is usually appropriate. true to copy the current System.MarshalByRefObject object's 

    identity to its clone,which will cause remoting client calls to be routed to the remote server 

    object.

  

   Returns: A shallow copy of the current System.MarshalByRefObject object.

  MemberwiseClone(self: object) -> object

  

   Creates a shallow copy of the current System.Object.

   Returns: A shallow copy of the current System.Object.
  """
  pass
 def Refresh(self):
  """
  Refresh(self: FileSystemInfo)

   Refreshes the state of the object.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,*args): #cannot find CLR constructor
  """
  __new__(cls: type)

  __new__(cls: type,info: SerializationInfo,context: StreamingContext)
  """
  pass
 def __reduce_ex__(self,*args):
  pass
 Attributes=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the attributes for the current file or directory.



Get: Attributes(self: FileSystemInfo) -> FileAttributes



Set: Attributes(self: FileSystemInfo)=value

"""

 CreationTime=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the creation time of the current file or directory.



Get: CreationTime(self: FileSystemInfo) -> DateTime



Set: CreationTime(self: FileSystemInfo)=value

"""

 CreationTimeUtc=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the creation time,in coordinated universal time (UTC),of the current file or directory.



Get: CreationTimeUtc(self: FileSystemInfo) -> DateTime



Set: CreationTimeUtc(self: FileSystemInfo)=value

"""

 Exists=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value indicating whether the file or directory exists.



Get: Exists(self: FileSystemInfo) -> bool



"""

 Extension=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the string representing the extension part of the file.



Get: Extension(self: FileSystemInfo) -> str



"""

 FullName=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the full path of the directory or file.



Get: FullName(self: FileSystemInfo) -> str



"""

 LastAccessTime=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the time the current file or directory was last accessed.



Get: LastAccessTime(self: FileSystemInfo) -> DateTime



Set: LastAccessTime(self: FileSystemInfo)=value

"""

 LastAccessTimeUtc=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the time,in coordinated universal time (UTC),that the current file or directory was last accessed.



Get: LastAccessTimeUtc(self: FileSystemInfo) -> DateTime



Set: LastAccessTimeUtc(self: FileSystemInfo)=value

"""

 LastWriteTime=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the time when the current file or directory was last written to.



Get: LastWriteTime(self: FileSystemInfo) -> DateTime



Set: LastWriteTime(self: FileSystemInfo)=value

"""

 LastWriteTimeUtc=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the time,in coordinated universal time (UTC),when the current file or directory was last written to.



Get: LastWriteTimeUtc(self: FileSystemInfo) -> DateTime



Set: LastWriteTimeUtc(self: FileSystemInfo)=value

"""

 Name=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """For files,gets the name of the file. For directories,gets the name of the last directory in the hierarchy if a hierarchy exists. Otherwise,the Name property gets the name of the directory.



Get: Name(self: FileSystemInfo) -> str



"""


 FullPath=None
 OriginalPath=None




class NumericUpDownAcceleration(object):
 """
 Provides information specifying how acceleration should be performed on a spin box (also known as an up-down control) when the up or down button is pressed for specified time period.

 

 NumericUpDownAcceleration(seconds: int,increment: Decimal)
 """
 @staticmethod
 def __new__(self,seconds,increment):
  """ __new__(cls: type,seconds: int,increment: Decimal) """
  pass
 Increment=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the quantity to increment or decrement the displayed value during acceleration.



Get: Increment(self: NumericUpDownAcceleration) -> Decimal



Set: Increment(self: NumericUpDownAcceleration)=value

"""

 Seconds=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the number of seconds the up or down button must be pressed before the acceleration starts.



Get: Seconds(self: NumericUpDownAcceleration) -> int



Set: Seconds(self: NumericUpDownAcceleration)=value

"""





"""Base Preflight test class that defines the minimum interface."""
# pylama:ignore=D401


class PreflightTestCase(object):
    """Base class for preflight tests."""

    def setUp(self, doc, output):
        """Hook method for setting up the test before exercising it."""
        pass

    def startTest(self, doc, output):
        """Hook method for exercising the test."""
        pass

    def tearDown(self, doc, output):
        """Hook method for deconstructing the test after testing it."""
        pass

    def doCleanups(self, doc, output):
        """Execute all cleanup functions. Normally called after tearDown."""
        pass



class BaseNumberConverter(TypeConverter):
 """ Provides a base type converter for nonfloating-point numerical types. """
 def CanConvertFrom(self,*__args):
  """
  CanConvertFrom(self: BaseNumberConverter,context: ITypeDescriptorContext,sourceType: Type) -> bool

  

   Determines if this converter can convert an object in the given source type to the native type 

    of the converter.

  

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context.

   sourceType: A System.Type that represents the type from which you want to convert.

   Returns: true if this converter can perform the operation; otherwise,false.
  """
  pass
 def CanConvertTo(self,*__args):
  """
  CanConvertTo(self: BaseNumberConverter,context: ITypeDescriptorContext,t: Type) -> bool

  

   Returns a value indicating whether this converter can convert an object to the given destination 

    type using the context.

  

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context.

   t: A System.Type that represents the type to which you want to convert.

   Returns: true if this converter can perform the operation; otherwise,false.
  """
  pass
 def ConvertFrom(self,*__args):
  """
  ConvertFrom(self: BaseNumberConverter,context: ITypeDescriptorContext,culture: CultureInfo,value: object) -> object

  

   Converts the given object to the converter's native type.

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context.

   culture: A System.Globalization.CultureInfo that specifies the culture to represent the number.

   value: The object to convert.

   Returns: An System.Object that represents the converted value.
  """
  pass
 def ConvertTo(self,*__args):
  """
  ConvertTo(self: BaseNumberConverter,context: ITypeDescriptorContext,culture: CultureInfo,value: object,destinationType: Type) -> object

  

   Converts the specified object to another type.

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context.

   culture: A System.Globalization.CultureInfo that specifies the culture to represent the number.

   value: The object to convert.

   destinationType: The type to convert the object to.

   Returns: An System.Object that represents the converted value.
  """
  pass



class BufferedGraphicsContext(object,IDisposable):
 """
 Provides methods for creating graphics buffers that can be used for double buffering.

 

 BufferedGraphicsContext()
 """
 def Allocate(self,*__args):
  """
  Allocate(self: BufferedGraphicsContext,targetDC: IntPtr,targetRectangle: Rectangle) -> BufferedGraphics

  

   Creates a graphics buffer of the specified size using the pixel format of the specified 

    System.Drawing.Graphics.

  

  

   targetDC: An System.IntPtr to a device context to match the pixel format of the new buffer to.

   targetRectangle: A System.Drawing.Rectangle indicating the size of the buffer to create.

   Returns: A System.Drawing.BufferedGraphics that can be used to draw to a buffer of the specified 

    dimensions.

  

  Allocate(self: BufferedGraphicsContext,targetGraphics: Graphics,targetRectangle: Rectangle) -> BufferedGraphics

  

   Creates a graphics buffer of the specified size using the pixel format of the specified 

    System.Drawing.Graphics.

  

  

   targetGraphics: The System.Drawing.Graphics to match the pixel format for the new buffer to.

   targetRectangle: A System.Drawing.Rectangle indicating the size of the buffer to create.

   Returns: A System.Drawing.BufferedGraphics that can be used to draw to a buffer of the specified 

    dimensions.
  """
  pass
 def Dispose(self):
  """
  Dispose(self: BufferedGraphicsContext)

   Releases all resources used by the System.Drawing.BufferedGraphicsContext.
  """
  pass
 def Invalidate(self):
  """
  Invalidate(self: BufferedGraphicsContext)

   Disposes of the current graphics buffer,if a buffer has been allocated and has not yet been 

    disposed.
  """
  pass
 def __enter__(self,*args):
  """
  __enter__(self: IDisposable) -> object

  

   Provides the implementation of __enter__ for objects which implement IDisposable.
  """
  pass
 def __exit__(self,*args):
  """
  __exit__(self: IDisposable,exc_type: object,exc_value: object,exc_back: object)

   Provides the implementation of __exit__ for objects which implement IDisposable.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __repr__(self,*args):
  """ __repr__(self: object) -> str """
  pass
 MaximumBuffer=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the maximum size of the buffer to use.



Get: MaximumBuffer(self: BufferedGraphicsContext) -> Size



Set: MaximumBuffer(self: BufferedGraphicsContext)=value

"""





class NumericUpDownAccelerationCollection(MarshalByRefObject,ICollection[NumericUpDownAcceleration],IEnumerable[NumericUpDownAcceleration],IEnumerable):
 """
 Represents a sorted collection of System.Windows.Forms.NumericUpDownAcceleration objects in the System.Windows.Forms.NumericUpDown control.

 

 NumericUpDownAccelerationCollection()
 """
 def Add(self,acceleration):
  """
  Add(self: NumericUpDownAccelerationCollection,acceleration: NumericUpDownAcceleration)

   Adds a new System.Windows.Forms.NumericUpDownAcceleration to the 

    System.Windows.Forms.NumericUpDownAccelerationCollection.

  

  

   acceleration: The System.Windows.Forms.NumericUpDownAcceleration to add to the 

    System.Windows.Forms.NumericUpDownAccelerationCollection.
  """
  pass
 def AddRange(self,accelerations):
  """
  AddRange(self: NumericUpDownAccelerationCollection,*accelerations: Array[NumericUpDownAcceleration])

   Adds the elements of the specified array to the 

    System.Windows.Forms.NumericUpDownAccelerationCollection,keeping the collection sorted.

  

  

   accelerations: An array of type System.Windows.Forms.NumericUpDownAcceleration  containing the objects to add 

    to the collection.
  """
  pass
 def Clear(self):
  """
  Clear(self: NumericUpDownAccelerationCollection)

   Removes all elements from the System.Windows.Forms.NumericUpDownAccelerationCollection.
  """
  pass
 def Contains(self,acceleration):
  """
  Contains(self: NumericUpDownAccelerationCollection,acceleration: NumericUpDownAcceleration) -> bool

  

   Determines whether the System.Windows.Forms.NumericUpDownAccelerationCollection contains a 

    specific System.Windows.Forms.NumericUpDownAcceleration.

  

  

   acceleration: The System.Windows.Forms.NumericUpDownAcceleration to locate in the 

    System.Windows.Forms.NumericUpDownAccelerationCollection.

  

   Returns: true if the System.Windows.Forms.NumericUpDownAcceleration is found in the 

    System.Windows.Forms.NumericUpDownAccelerationCollection; otherwise,false.
  """
  pass
 def CopyTo(self,array,index):
  """
  CopyTo(self: NumericUpDownAccelerationCollection,array: Array[NumericUpDownAcceleration],index: int)

   Copies the System.Windows.Forms.NumericUpDownAccelerationCollection values to a one-dimensional 

    System.Windows.Forms.NumericUpDownAcceleration instance at the specified index.

  

  

   array: The one-dimensional System.Windows.Forms.NumericUpDownAcceleration that is the destination of 

    the values copied from System.Windows.Forms.NumericUpDownAccelerationCollection.

  

   index: The index in array where copying begins.
  """
  pass
 def MemberwiseClone(self,*args):
  """
  MemberwiseClone(self: MarshalByRefObject,cloneIdentity: bool) -> MarshalByRefObject

  

   Creates a shallow copy of the current System.MarshalByRefObject object.

  

   cloneIdentity: false to delete the current System.MarshalByRefObject object's identity,which will cause the 

    object to be assigned a new identity when it is marshaled across a remoting boundary. A value of 

    false is usually appropriate. true to copy the current System.MarshalByRefObject object's 

    identity to its clone,which will cause remoting client calls to be routed to the remote server 

    object.

  

   Returns: A shallow copy of the current System.MarshalByRefObject object.

  MemberwiseClone(self: object) -> object

  

   Creates a shallow copy of the current System.Object.

   Returns: A shallow copy of the current System.Object.
  """
  pass
 def Remove(self,acceleration):
  """
  Remove(self: NumericUpDownAccelerationCollection,acceleration: NumericUpDownAcceleration) -> bool

  

   Removes the first occurrence of the specified System.Windows.Forms.NumericUpDownAcceleration 

    from the System.Windows.Forms.NumericUpDownAccelerationCollection.

  

  

   acceleration: The System.Windows.Forms.NumericUpDownAcceleration to remove from the collection.

   Returns: true if the System.Windows.Forms.NumericUpDownAcceleration is removed from 

    System.Windows.Forms.NumericUpDownAccelerationCollection; otherwise,false.
  """
  pass
 def __add__(self,*args):
  """ x.__add__(y) <==> x+y """
  pass
 def __contains__(self,*args):
  """ __contains__(self: ICollection[NumericUpDownAcceleration],item: NumericUpDownAcceleration) -> bool """
  pass
 def __getitem__(self,*args):
  """ x.__getitem__(y) <==> x[y] """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __iter__(self,*args):
  """ __iter__(self: IEnumerable) -> object """
  pass
 def __len__(self,*args):
  """ x.__len__() <==> len(x) """
  pass
 Count=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the number of objects in the System.Windows.Forms.NumericUpDownAccelerationCollection.



Get: Count(self: NumericUpDownAccelerationCollection) -> int



"""

 IsReadOnly=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value indicating whether the System.Windows.Forms.NumericUpDownAccelerationCollection is read-only.



Get: IsReadOnly(self: NumericUpDownAccelerationCollection) -> bool



"""





class IEditableObject:
 """ Provides functionality to commit or rollback changes to an object that is used as a data source. """
 def BeginEdit(self):
  """
  BeginEdit(self: IEditableObject)

   Begins an edit on an object.
  """
  pass
 def CancelEdit(self):
  """
  CancelEdit(self: IEditableObject)

   Discards changes since the last System.ComponentModel.IEditableObject.BeginEdit call.
  """
  pass
 def EndEdit(self):
  """
  EndEdit(self: IEditableObject)

   Pushes changes since the last System.ComponentModel.IEditableObject.BeginEdit or 

    System.ComponentModel.IBindingList.AddNew call into the underlying object.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass



class FileAttributes(Enum,IComparable,IFormattable,IConvertible):
 """
 Provides attributes for files and directories.

 

 enum (flags) FileAttributes,values: Archive (32),Compressed (2048),Device (64),Directory (16),Encrypted (16384),Hidden (2),IntegrityStream (32768),Normal (128),NoScrubData (131072),NotContentIndexed (8192),Offline (4096),ReadOnly (1),ReparsePoint (1024),SparseFile (512),System (4),Temporary (256)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 Archive=None
 Compressed=None
 Device=None
 Directory=None
 Encrypted=None
 Hidden=None
 IntegrityStream=None
 Normal=None
 NoScrubData=None
 NotContentIndexed=None
 Offline=None
 ReadOnly=None
 ReparsePoint=None
 SparseFile=None
 System=None
 Temporary=None
 value__=None




# coding=utf-8
"""Wrapping Autodesk Desktop Connector API."""

import os.path as op
from pyrevit import PyRevitException
from pyrevit.framework import clr, Process
from pyrevit.coreutils import logger

from pyrevit import HOST_APP

mlogger = logger.get_logger(__name__)

ADC_NAME = "Autodesk Desktop Connector"
ADC_SHORTNAME = "ADC"
ADC_DRIVE_SCHEMA = "{drive_name}://"

if HOST_APP.is_newer_than("2024"):  # .net8 core version of the desktop connector API
    ADC_DEFAULT_INSTALL_PATH = (
        r"C:\Program Files\Autodesk\Desktop Connector\FOS\AddInProcess\Civil3DOE"
    )
else:  # .net framework version of the desktop connector API
    ADC_DEFAULT_INSTALL_PATH = r"C:\Program Files\Autodesk\Desktop Connector"

ADC_API_DLL = "Autodesk.DesktopConnector.API.dll"
ADC_API_DLL_PATH = op.join(ADC_DEFAULT_INSTALL_PATH, ADC_API_DLL)

API = None
if op.exists(ADC_API_DLL_PATH):
    try:
        clr.AddReferenceToFileAndPath(ADC_API_DLL_PATH)
        import Autodesk.DesktopConnector.API as API  # pylint: disable=import-error
    except:
        pass


def _ensure_api():
    if not API:
        raise PyRevitException("{} is not loaded".format(ADC_NAME))
    return API


def _get_all_processids():
    return [x.Id for x in Process.GetProcesses()]


def _get_adc():
    api = _ensure_api()
    return api.DesktopConnectorService()


def _get_drives_info(adc):
    return list(adc.GetDrives())


def _get_drive_properties(adc, drive):
    return adc.GetPropertyDefinitions(drive.Id)


def _get_drive_from_path(adc, path):
    for drv_info in _get_drives_info(adc):
        drive_schema = ADC_DRIVE_SCHEMA.format(drive_name=drv_info.Name)
        if path.lower().startswith(drive_schema.lower()):
            return drv_info


def _get_drive_from_local_path(adc, local_path):
    for drv_info in _get_drives_info(adc):
        drv_localpath = op.normpath(drv_info.WorkspaceLocation)
        if op.normpath(local_path).startswith(drv_localpath):
            return drv_info


def _drive_path_to_local_path(drv_info, path):
    drive_schema = ADC_DRIVE_SCHEMA.format(drive_name=drv_info.Name)
    return op.normpath(
        op.join(drv_info.WorkspaceLocation, path.replace(drive_schema, ""))
    )


def _ensure_local_path(adc, path):
    drv_info = _get_drive_from_path(adc, path)
    if drv_info:
        return _drive_path_to_local_path(drv_info, path)
    elif not _get_drive_from_local_path(adc, path):
        raise PyRevitException("Path is not inside any ADC drive")
    return path


def _get_item(adc, path):
    path = _ensure_local_path(adc, path)
    if not op.isfile(path):
        raise PyRevitException("Path does not point to a file")

    res = adc.GetItemsByWorkspacePaths([path])
    if not res:
        raise PyRevitException("Can not find item in any ADC drive")
    # grab the first item (we only except one since path is to a file)
    return res[0].Item


def _get_item_drive(adc, item):
    for drv_info in _get_drives_info(adc):
        if drv_info.Id == item.DriveId:
            return drv_info


def _get_item_lockstatus(adc, item):
    res = adc.GetLockStatus([item.Id])
    if res and res.Status:
        return res.Status[0]


def _get_item_property_value(adc, drive, item, prop_name):
    for prop_def in _get_drive_properties(adc, drive):
        if prop_def.DisplayName == prop_name:
            res = adc.GetProperties([item.Id], [prop_def.Id])
            if res:
                return res.Values[0]


def _get_item_property_id_value(adc, drive, item, prop_id):
    for prop_def in _get_drive_properties(adc, drive):
        if prop_def.Id == prop_id:
            res = adc.GetProperties([item.Id], [prop_def.Id])
            if res:
                return res.Values[0]


def is_available():
    """Check if ADC service is available."""
    try:
        _get_adc().Discover()
        return True
    except Exception:
        return False


def get_drive_paths():
    """Get dict of local paths for ADC drives."""
    adc = _get_adc()
    return {x.Name: x.WorkspaceLocation for x in _get_drives_info(adc)}


def get_local_path(path):
    """Convert ADC BIM360 drive path to local path."""
    adc = _get_adc()
    drv_info = _get_drive_from_path(adc, path)
    if drv_info:
        return _drive_path_to_local_path(drv_info, path)


def lock_file(path):
    """Lock given file."""
    adc = _get_adc()
    item = _get_item(adc, path)
    adc.LockFile(item.Id)


def is_locked(path):
    """Check if file is locked."""
    adc = _get_adc()
    item = _get_item(adc, path)
    lock_status = _get_item_lockstatus(adc, item)
    return lock_status.State == API.LockState.LockedByOther, lock_status.LockOwner


def unlock_file(path):
    """Unlock given file."""
    adc = _get_adc()
    item = _get_item(adc, path)
    adc.UnlockFile(item.Id)


def is_synced(path):
    """Check if file is synchronized."""
    adc = _get_adc()
    item = _get_item(adc, path)
    drive = _get_item_drive(adc, item)
    # ADC uses translated property names so
    # check status property by its type "LocalState"
    # see https://github.com/pyrevitlabs/pyRevit/issues/1152
    # ADC version 15 changed property_id_value
    # see https://github.com/pyrevitlabs/pyRevit/issues/1371
    prop_val = _get_item_property_id_value(
        adc, drive, item, "DesktopConnector.Core.LocalState"
    )
    if prop_val is None:
        # version older than ADC 15
        prop_val = _get_item_property_id_value(adc, drive, item, "LocalState")
    # possible values, 'Cached', 'Stale', 'Modified'
    # .Value is not translated
    return prop_val.Value == "Cached" or prop_val.Value == "Synced"


def sync_file(path, force=False):
    """Force ADC to sync given file to latest version."""
    if not force and is_synced(path):
        return
    adc = _get_adc()
    item = _get_item(adc, path)
    # make sure path is local
    local_path = _ensure_local_path(adc, path)
    for proc_id in _get_all_processids():
        adc.FileClosedWithinRunningProcess(proc_id, local_path)
    adc.SyncFiles([item.Id])



class TrackBarRenderer(object):
 """ Provides methods used to render a track bar control with visual styles. This class cannot be inherited. """
 @staticmethod
 def DrawBottomPointingThumb(g,bounds,state):
  """
  DrawBottomPointingThumb(g: Graphics,bounds: Rectangle,state: TrackBarThumbState)

   Draws a downward-pointing track bar slider (also known as the thumb) with visual styles.

  

   g: The System.Drawing.Graphics used to draw the track bar slider.

   bounds: The System.Drawing.Rectangle that specifies the bounds of the track bar slider.

   state: One of the System.Windows.Forms.VisualStyles.TrackBarThumbState values that specifies the visual 

    state of the track bar slider.
  """
  pass
 @staticmethod
 def DrawHorizontalThumb(g,bounds,state):
  """
  DrawHorizontalThumb(g: Graphics,bounds: Rectangle,state: TrackBarThumbState)

   Draws a horizontal track bar slider (also known as the thumb) with visual styles.

  

   g: The System.Drawing.Graphics used to draw the track bar slider.

   bounds: The System.Drawing.Rectangle that specifies the bounds of the track bar slider.

   state: One of the System.Windows.Forms.VisualStyles.TrackBarThumbState values that specifies the visual 

    state of the track bar slider.
  """
  pass
 @staticmethod
 def DrawHorizontalTicks(g,bounds,numTicks,edgeStyle):
  """
  DrawHorizontalTicks(g: Graphics,bounds: Rectangle,numTicks: int,edgeStyle: EdgeStyle)

   Draws the specified number of horizontal track bar ticks with visual styles.

  

   g: The System.Drawing.Graphics used to draw the ticks.

   bounds: The System.Drawing.Rectangle that specifies the bounds of the ticks.

   numTicks: The number of ticks to draw.

   edgeStyle: One of the System.Windows.Forms.VisualStyles.EdgeStyle values.
  """
  pass
 @staticmethod
 def DrawHorizontalTrack(g,bounds):
  """
  DrawHorizontalTrack(g: Graphics,bounds: Rectangle)

   Draws the track for a horizontal track bar with visual styles.

  

   g: The System.Drawing.Graphics used to draw the track.

   bounds: The System.Drawing.Rectangle that specifies the bounds of the track.
  """
  pass
 @staticmethod
 def DrawLeftPointingThumb(g,bounds,state):
  """
  DrawLeftPointingThumb(g: Graphics,bounds: Rectangle,state: TrackBarThumbState)

   Draws a left-pointing track bar slider (also known as the thumb) with visual styles.

  

   g: The System.Drawing.Graphics used to draw the track bar slider.

   bounds: The System.Drawing.Rectangle that specifies the bounds of the track bar slider.

   state: One of the System.Windows.Forms.VisualStyles.TrackBarThumbState values that specifies the visual 

    state of the track bar slider.
  """
  pass
 @staticmethod
 def DrawRightPointingThumb(g,bounds,state):
  """
  DrawRightPointingThumb(g: Graphics,bounds: Rectangle,state: TrackBarThumbState)

   Draws a right-pointing track bar slider (also known as the thumb) with visual styles.

  

   g: The System.Drawing.Graphics used to draw the track bar slider.

   bounds: The System.Drawing.Rectangle that specifies the bounds of the track bar slider.

   state: One of the System.Windows.Forms.VisualStyles.TrackBarThumbState values that specifies the visual 

    state of the track bar slider.
  """
  pass
 @staticmethod
 def DrawTopPointingThumb(g,bounds,state):
  """
  DrawTopPointingThumb(g: Graphics,bounds: Rectangle,state: TrackBarThumbState)

   Draws an upward-pointing track bar slider (also known as the thumb) with visual styles.

  

   g: The System.Drawing.Graphics used to draw the track bar slider.

   bounds: The System.Drawing.Rectangle that specifies the bounds of the track bar slider.

   state: One of the System.Windows.Forms.VisualStyles.TrackBarThumbState values that specifies the visual 

    state of the track bar slider.
  """
  pass
 @staticmethod
 def DrawVerticalThumb(g,bounds,state):
  """
  DrawVerticalThumb(g: Graphics,bounds: Rectangle,state: TrackBarThumbState)

   Draws a vertical track bar slider (also known as the thumb) with visual styles.

  

   g: The System.Drawing.Graphics used to draw the track bar slider.

   bounds: The System.Drawing.Rectangle that specifies the bounds of the track bar slider.

   state: One of the System.Windows.Forms.VisualStyles.TrackBarThumbState values that specifies the visual 

    state of the track bar slider.
  """
  pass
 @staticmethod
 def DrawVerticalTicks(g,bounds,numTicks,edgeStyle):
  """
  DrawVerticalTicks(g: Graphics,bounds: Rectangle,numTicks: int,edgeStyle: EdgeStyle)

   Draws the specified number of vertical track bar ticks with visual styles.

  

   g: The System.Drawing.Graphics used to draw the ticks.

   bounds: The System.Drawing.Rectangle that specifies the bounds of the ticks.

   numTicks: The number of ticks to draw.

   edgeStyle: One of the System.Windows.Forms.VisualStyles.EdgeStyle values.
  """
  pass
 @staticmethod
 def DrawVerticalTrack(g,bounds):
  """
  DrawVerticalTrack(g: Graphics,bounds: Rectangle)

   Draws the track for a vertical track bar with visual styles.

  

   g: The System.Drawing.Graphics used to draw the track.

   bounds: The System.Drawing.Rectangle that specifies the bounds of the track.
  """
  pass
 @staticmethod
 def GetBottomPointingThumbSize(g,state):
  """
  GetBottomPointingThumbSize(g: Graphics,state: TrackBarThumbState) -> Size

  

   Returns the size,in pixels,of the track bar slider (also known as the thumb) that points down.

  

   g: The System.Drawing.Graphics this operation will use.

   state: One of the System.Windows.Forms.VisualStyles.TrackBarThumbState values that specifies the visual 

    state of the track bar slider.

  

   Returns: A System.Drawing.Size that specifies the size,in pixels,of the slider.
  """
  pass
 @staticmethod
 def GetLeftPointingThumbSize(g,state):
  """
  GetLeftPointingThumbSize(g: Graphics,state: TrackBarThumbState) -> Size

  

   Returns the size,in pixels,of the track bar slider (also known as the thumb) that points to 

    the left.

  

  

   g: The System.Drawing.Graphics this operation will use.

   state: One of the System.Windows.Forms.VisualStyles.TrackBarThumbState values that specifies the visual 

    state of the slider.

  

   Returns: A System.Drawing.Size that specifies the size,in pixels,of the slider.
  """
  pass
 @staticmethod
 def GetRightPointingThumbSize(g,state):
  """
  GetRightPointingThumbSize(g: Graphics,state: TrackBarThumbState) -> Size

  

   Returns the size,in pixels,of the track bar slider (also known as the thumb) that points to 

    the right.

  

  

   g: The System.Drawing.Graphics this operation will use.

   state: One of the System.Windows.Forms.VisualStyles.TrackBarThumbState values that specifies the visual 

    state of the slider.

  

   Returns: A System.Drawing.Size that specifies the size,in pixels,of the slider.
  """
  pass
 @staticmethod
 def GetTopPointingThumbSize(g,state):
  """
  GetTopPointingThumbSize(g: Graphics,state: TrackBarThumbState) -> Size

  

   Returns the size,in pixels,of the track bar slider (also known as the thumb) that points up.

  

   g: The System.Drawing.Graphics this operation will use.

   state: One of the System.Windows.Forms.VisualStyles.TrackBarThumbState values that specifies the visual 

    state of the slider.

  

   Returns: A System.Drawing.Size that specifies the size,in pixels,of the slider.
  """
  pass
 IsSupported=True




class UnmanagedType(Enum,IComparable,IFormattable,IConvertible):
 """
 Identifies how to marshal parameters or fields to unmanaged code.

 

 enum UnmanagedType,values: AnsiBStr (35),AsAny (40),Bool (2),BStr (19),ByValArray (30),ByValTStr (23),Currency (15),CustomMarshaler (44),Error (45),FunctionPtr (38),HString (47),I1 (3),I2 (5),I4 (7),I8 (9),IDispatch (26),IInspectable (46),Interface (28),IUnknown (25),LPArray (42),LPStr (20),LPStruct (43),LPTStr (22),LPUTF8Str (48),LPWStr (21),R4 (11),R8 (12),SafeArray (29),Struct (27),SysInt (31),SysUInt (32),TBStr (36),U1 (4),U2 (6),U4 (8),U8 (10),VariantBool (37),VBByRefStr (34)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 AnsiBStr=None
 AsAny=None
 Bool=None
 BStr=None
 ByValArray=None
 ByValTStr=None
 Currency=None
 CustomMarshaler=None
 Error=None
 FunctionPtr=None
 HString=None
 I1=None
 I2=None
 I4=None
 I8=None
 IDispatch=None
 IInspectable=None
 Interface=None
 IUnknown=None
 LPArray=None
 LPStr=None
 LPStruct=None
 LPTStr=None
 LPUTF8Str=None
 LPWStr=None
 R4=None
 R8=None
 SafeArray=None
 Struct=None
 SysInt=None
 SysUInt=None
 TBStr=None
 U1=None
 U2=None
 U4=None
 U8=None
 value__=None
 VariantBool=None
 VBByRefStr=None




class DataGridViewSelectedColumnCollection(BaseCollection,ICollection,IEnumerable,IList):
 """ Represents a collection of System.Windows.Forms.DataGridViewColumn objects that are selected in a System.Windows.Forms.DataGridView. """
 def Clear(self):
  """
  Clear(self: DataGridViewSelectedColumnCollection)

   Clears the collection.
  """
  pass
 def Contains(self,dataGridViewColumn):
  """
  Contains(self: DataGridViewSelectedColumnCollection,dataGridViewColumn: DataGridViewColumn) -> bool

  

   Determines whether the specified column is contained in the collection.

  

   dataGridViewColumn: A System.Windows.Forms.DataGridViewColumn to locate in the 

    System.Windows.Forms.DataGridViewSelectedColumnCollection.

  

   Returns: true if the dataGridViewColumn parameter is in the collection; otherwise,false.
  """
  pass
 def CopyTo(self,*__args):
  """
  CopyTo(self: DataGridViewSelectedColumnCollection,array: Array[DataGridViewColumn],index: int)

   Copies the elements of the collection to the specified array,starting at the specified index.

  

   array: The one-dimensional array that is the destination of the elements copied from the collection. 

    The array must have zero-based indexing.

  

   index: The zero-based index in the array at which copying begins.
  """
  pass
 def Insert(self,index,dataGridViewColumn):
  """
  Insert(self: DataGridViewSelectedColumnCollection,index: int,dataGridViewColumn: DataGridViewColumn)

   Inserts a column into the collection at the specified position.

  

   index: The zero-based index at which the column should be inserted.

   dataGridViewColumn: The System.Windows.Forms.DataGridViewColumn to insert into the 

    System.Windows.Forms.DataGridViewSelectedColumnCollection.
  """
  pass
 def MemberwiseClone(self,*args):
  """
  MemberwiseClone(self: MarshalByRefObject,cloneIdentity: bool) -> MarshalByRefObject

  

   Creates a shallow copy of the current System.MarshalByRefObject object.

  

   cloneIdentity: false to delete the current System.MarshalByRefObject object's identity,which will cause the 

    object to be assigned a new identity when it is marshaled across a remoting boundary. A value of 

    false is usually appropriate. true to copy the current System.MarshalByRefObject object's 

    identity to its clone,which will cause remoting client calls to be routed to the remote server 

    object.

  

   Returns: A shallow copy of the current System.MarshalByRefObject object.

  MemberwiseClone(self: object) -> object

  

   Creates a shallow copy of the current System.Object.

   Returns: A shallow copy of the current System.Object.
  """
  pass
 def __contains__(self,*args):
  """
  __contains__(self: IList,value: object) -> bool

  

   Determines whether the System.Collections.IList contains a specific value.

  

   value: The object to locate in the System.Collections.IList.

   Returns: true if the System.Object is found in the System.Collections.IList; otherwise,false.
  """
  pass
 def __getitem__(self,*args):
  """ x.__getitem__(y) <==> x[y] """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __iter__(self,*args):
  """ __iter__(self: IEnumerable) -> object """
  pass
 def __len__(self,*args):
  """ x.__len__() <==> len(x) """
  pass
 List=property(lambda self: object(),lambda self,v: None,lambda self: None)





class UriPartial(Enum,IComparable,IFormattable,IConvertible):
 """
 Defines the parts of a URI for the System.Uri.GetLeftPart(System.UriPartial) method.

 

 enum UriPartial,values: Authority (1),Path (2),Query (3),Scheme (0)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 Authority=None
 Path=None
 Query=None
 Scheme=None
 value__=None




class MulticastDelegate(Delegate,ICloneable,ISerializable):
 """ Represents a multicast delegate; that is,a delegate that can have more than one element in its invocation list. """
 def CombineImpl(self,*args):
  """
  CombineImpl(self: MulticastDelegate,follow: Delegate) -> Delegate

  

   Combines this System.Delegate with the specified System.Delegate to form a new delegate.

  

   follow: The delegate to combine with this delegate.

   Returns: A delegate that is the new root of the System.MulticastDelegate invocation list.
  """
  pass
 def DynamicInvokeImpl(self,*args):
  """
  DynamicInvokeImpl(self: Delegate,args: Array[object]) -> object

  

   Dynamically invokes (late-bound) the method represented by the current delegate.

  

   args: An array of objects that are the arguments to pass to the method represented by the current 

    delegate.-or- null,if the method represented by the current delegate does not require 

    arguments.

  

   Returns: The object returned by the method represented by the delegate.
  """
  pass
 def Equals(self,obj):
  """
  Equals(self: MulticastDelegate,obj: object) -> bool

  

   Determines whether this multicast delegate and the specified object are equal.

  

   obj: The object to compare with this instance.

   Returns: true if obj and this instance have the same invocation lists; otherwise,false.
  """
  pass
 def GetHashCode(self):
  """
  GetHashCode(self: MulticastDelegate) -> int

  

   Returns the hash code for this instance.

   Returns: A 32-bit signed integer hash code.
  """
  pass
 def GetInvocationList(self):
  """
  GetInvocationList(self: MulticastDelegate) -> Array[Delegate]

  

   Returns the invocation list of this multicast delegate,in invocation order.

   Returns: An array of delegates whose invocation lists collectively match the invocation list of this 

    instance.
  """
  pass
 def GetMethodImpl(self,*args):
  """
  GetMethodImpl(self: MulticastDelegate) -> MethodInfo

  

   Returns a static method represented by the current System.MulticastDelegate.

   Returns: A static method represented by the current System.MulticastDelegate.
  """
  pass
 def GetObjectData(self,info,context):
  """
  GetObjectData(self: MulticastDelegate,info: SerializationInfo,context: StreamingContext)

   Populates a System.Runtime.Serialization.SerializationInfo object with all the data needed to 

    serialize this instance.

  

  

   info: An object that holds all the data needed to serialize or deserialize this instance.

   context: (Reserved) The location where serialized data is stored and retrieved.
  """
  pass
 def RemoveImpl(self,*args):
  """
  RemoveImpl(self: MulticastDelegate,value: Delegate) -> Delegate

  

   Removes an element from the invocation list of this System.MulticastDelegate that is equal to 

    the specified delegate.

  

  

   value: The delegate to search for in the invocation list.

   Returns: If value is found in the invocation list for this instance,then a new System.Delegate without 

    value in its invocation list; otherwise,this instance with its original invocation list.
  """
  pass
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==y """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,*args): #cannot find CLR constructor
  """
  __new__(cls: type,target: object,method: str)

  __new__(cls: type,target: Type,method: str)
  """
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass



class DragDeltaEventArgs(RoutedEventArgs):
 """
 Provides information about the System.Windows.Controls.Primitives.Thumb.DragDelta event that occurs one or more times when a user drags a System.Windows.Controls.Primitives.Thumb control with the mouse.

 

 DragDeltaEventArgs(horizontalChange: float,verticalChange: float)
 """
 @staticmethod
 def __new__(self,horizontalChange,verticalChange):
  """ __new__(cls: type,horizontalChange: float,verticalChange: float) """
  pass
 HorizontalChange=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the horizontal distance that the mouse has moved since the previous System.Windows.Controls.Primitives.Thumb.DragDelta event when the user drags the System.Windows.Controls.Primitives.Thumb control with the mouse.



Get: HorizontalChange(self: DragDeltaEventArgs) -> float



"""

 VerticalChange=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the vertical distance that the mouse has moved since the previous System.Windows.Controls.Primitives.Thumb.DragDelta event when the user drags the System.Windows.Controls.Primitives.Thumb with the mouse.



Get: VerticalChange(self: DragDeltaEventArgs) -> float



"""





class DataGridPreparingCellForEditEventArgs(EventArgs):
 """
 Provides data for the System.Windows.Controls.DataGrid.PreparingCellForEdit event.

 

 DataGridPreparingCellForEditEventArgs(column: DataGridColumn,row: DataGridRow,editingEventArgs: RoutedEventArgs,editingElement: FrameworkElement)
 """
 @staticmethod
 def __new__(self,column,row,editingEventArgs,editingElement):
  """ __new__(cls: type,column: DataGridColumn,row: DataGridRow,editingEventArgs: RoutedEventArgs,editingElement: FrameworkElement) """
  pass
 Column=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the column that contains the cell to be edited.



Get: Column(self: DataGridPreparingCellForEditEventArgs) -> DataGridColumn



"""

 EditingElement=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the element that the column displays for a cell in editing mode.



Get: EditingElement(self: DataGridPreparingCellForEditEventArgs) -> FrameworkElement



"""

 EditingEventArgs=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets information about the user gesture that caused the cell to enter edit mode.



Get: EditingEventArgs(self: DataGridPreparingCellForEditEventArgs) -> RoutedEventArgs



"""

 Row=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the row that contains the cell to be edited.



Get: Row(self: DataGridPreparingCellForEditEventArgs) -> DataGridRow



"""





"""Handle creation of output window helper links."""

from pyrevit.compat import safe_strtype, get_elementid_value_func
from pyrevit.coreutils.logger import get_logger


#pylint: disable=W0703,C0302,C0103
mlogger = get_logger(__name__)


PROTOCOL_NAME = 'revit://outputhelpers?'
LINK_SHOW_ICON = \
    'M9.5,3A6.5,6.5 0 0,1 16,9.5C16,11.11 15.41,12.59 14.44,13.73L14.71,' \
    '14H15.5L20.5,19L19,20.5L14,15.5V14.71L13.73,14.44C12.59,15.41 11.11,' \
    '16 9.5,16A6.5,6.5 0 0,1 3,9.5A6.5,6.5 0 0,1 9.5,3M9.5,5C7,5 5,7 5,' \
    '9.5C5,12 7,14 9.5,14C12,14 14,12 14,9.5C14,7 12,5 9.5,5Z'
DEFAULT_LINK = '<span>' \
               '<a class="elementlink" ' \
                  'style="margin-right:0px;padding-right:3px" ' \
                  'title="Click to select or show element" ' \
                  '{attrs_select} >' \
                     '{ids}' \
               '</a>' \
               '<a class="elementlink" ' \
                  'style="margin-left:0px;padding-left:3px" ' \
                  '{attrs_show}>' \
                     '<svg width="14" height="14" viewBox="2 2 20 20" >' \
                        '<path fill="white" d="{show_icon}" />' \
                     '</svg>' \
               '</a>' \
               '</span>'


def make_link(element_ids, contents=None):
    """Create link for given element ids.

    This link is a special format link with revit:// scheme that is handled
    by the output window to select the provided element ids in current project.
    Scripts should not call this function directly. Creating clickable element
    links is handled by the output wrapper object through the :func:`linkify`
    method.

    Examples:
        ```python
        output = pyrevit.output.get_output()
        for idx, elid in enumerate(element_ids):
            print('{}: {}'.format(idx+1, output.linkify(elid)))
        ```
    """
    get_elementid_value = get_elementid_value_func()
    try:
        try:
            strids = [safe_strtype(get_elementid_value(x)) for x in element_ids]
        except TypeError:
            strids = [safe_strtype(get_elementid_value(element_ids))]
    except AttributeError:
        raise ValueError("One or more items are not ElementIds")

    elementquery = ('element[]={}'.format(strid) for strid in strids)

    reviturl = '&'.join(elementquery)
    link_title = ', '.join(strids)

    if len(reviturl) >= 2000:
        total_length = 0
        max_elements = 0
        elementquery_parts = ['element[]=' + strid for strid in strids]
        for i, part in enumerate(elementquery_parts):
            if i > 0:
                total_length += 1  # for '&'
            total_length += len(part)
            if total_length >= 2000:
                break
            max_elements += 1

        alertjs = \
            'alert(&quot;URL was too long ({} characters). Maximum allowed is 2000. '\
            'Only {} out of {} elements could fit.&quot;);'\
            .format(len(reviturl), max_elements, len(strids))
        linkattrs_select = 'href="#" onClick="{}"'.format(alertjs)
        linkattrs_show = linkattrs_select
    else:
        base_link = 'href="{}{}{}&show={{}}"'.format(
            PROTOCOL_NAME, '&command=select&', reviturl
        )
        linkattrs_select = base_link.format("false")
        linkattrs_show = base_link.format("true")

    return DEFAULT_LINK.format(
        attrs_select=linkattrs_select,
        attrs_show=linkattrs_show,
        ids=contents or link_title,
        show_icon=LINK_SHOW_ICON
    )



class Stream(MarshalByRefObject,IDisposable):
 """ Provides a generic view of a sequence of bytes. """
 def BeginRead(self,buffer,offset,count,callback,state):
  """
  BeginRead(self: Stream,buffer: Array[Byte],offset: int,count: int,callback: AsyncCallback,state: object) -> IAsyncResult

  

   Begins an asynchronous read operation.

  

   buffer: The buffer to read the data into.

   offset: The byte offset in buffer at which to begin writing data read from the stream.

   count: The maximum number of bytes to read.

   callback: An optional asynchronous callback,to be called when the read is complete.

   state: A user-provided object that distinguishes this particular asynchronous read request from other 

    requests.

  

   Returns: An System.IAsyncResult that represents the asynchronous read,which could still be pending.
  """
  pass
 def BeginWrite(self,buffer,offset,count,callback,state):
  """
  BeginWrite(self: Stream,buffer: Array[Byte],offset: int,count: int,callback: AsyncCallback,state: object) -> IAsyncResult

  

   Begins an asynchronous write operation.

  

   buffer: The buffer to write data from.

   offset: The byte offset in buffer from which to begin writing.

   count: The maximum number of bytes to write.

   callback: An optional asynchronous callback,to be called when the write is complete.

   state: A user-provided object that distinguishes this particular asynchronous write request from other 

    requests.

  

   Returns: An IAsyncResult that represents the asynchronous write,which could still be pending.
  """
  pass
 def Close(self):
  """
  Close(self: Stream)

   Closes the current stream and releases any resources (such as sockets and file handles) 

    associated with the current stream.
  """
  pass
 def CopyTo(self,destination,bufferSize=None):
  """
  CopyTo(self: Stream,destination: Stream,bufferSize: int)

   Reads all the bytes from the current stream and writes them to a destination stream,using a 

    specified buffer size.

  

  

   destination: The stream that will contain the contents of the current stream.

   bufferSize: The size of the buffer. This value must be greater than zero. The default size is 4096.

  CopyTo(self: Stream,destination: Stream)

   Reads the bytes from the current stream and writes them to the destination stream.

  

   destination: The stream that will contain the contents of the current stream.
  """
  pass
 def CopyToAsync(self,destination,bufferSize=None,cancellationToken=None):
  """
  CopyToAsync(self: Stream,destination: Stream,bufferSize: int,cancellationToken: CancellationToken) -> Task

  CopyToAsync(self: Stream,destination: Stream,bufferSize: int) -> Task

  CopyToAsync(self: Stream,destination: Stream) -> Task
  """
  pass
 def CreateWaitHandle(self,*args):
  """
  CreateWaitHandle(self: Stream) -> WaitHandle

  

   Allocates a System.Threading.WaitHandle object.

   Returns: A reference to the allocated WaitHandle.
  """
  pass
 def Dispose(self):
  """
  Dispose(self: Stream)

   Releases all resources used by the System.IO.Stream.
  """
  pass
 def EndRead(self,asyncResult):
  """
  EndRead(self: Stream,asyncResult: IAsyncResult) -> int

  

   Waits for the pending asynchronous read to complete.

  

   asyncResult: The reference to the pending asynchronous request to finish.

   Returns: The number of bytes read from the stream,between zero (0) and the number of bytes you 

    requested. Streams return zero (0) only at the end of the stream,otherwise,they should block 

    until at least one byte is available.
  """
  pass
 def EndWrite(self,asyncResult):
  """
  EndWrite(self: Stream,asyncResult: IAsyncResult)

   Ends an asynchronous write operation.

  

   asyncResult: A reference to the outstanding asynchronous I/O request.
  """
  pass
 def Flush(self):
  """
  Flush(self: Stream)

   When overridden in a derived class,clears all buffers for this stream and causes any buffered 

    data to be written to the underlying device.
  """
  pass
 def FlushAsync(self,cancellationToken=None):
  """
  FlushAsync(self: Stream,cancellationToken: CancellationToken) -> Task

  FlushAsync(self: Stream) -> Task
  """
  pass
 def MemberwiseClone(self,*args):
  """
  MemberwiseClone(self: MarshalByRefObject,cloneIdentity: bool) -> MarshalByRefObject

  

   Creates a shallow copy of the current System.MarshalByRefObject object.

  

   cloneIdentity: false to delete the current System.MarshalByRefObject object's identity,which will cause the 

    object to be assigned a new identity when it is marshaled across a remoting boundary. A value of 

    false is usually appropriate. true to copy the current System.MarshalByRefObject object's 

    identity to its clone,which will cause remoting client calls to be routed to the remote server 

    object.

  

   Returns: A shallow copy of the current System.MarshalByRefObject object.

  MemberwiseClone(self: object) -> object

  

   Creates a shallow copy of the current System.Object.

   Returns: A shallow copy of the current System.Object.
  """
  pass
 def ObjectInvariant(self,*args):
  """
  ObjectInvariant(self: Stream)

   Provides support for a System.Diagnostics.Contracts.Contract.
  """
  pass
 def Read(self,buffer,offset,count):
  """
  Read(self: Stream,offset: int,count: int) -> (int,Array[Byte])

  

   When overridden in a derived class,reads a sequence of bytes from the current stream and 

    advances the position within the stream by the number of bytes read.

  

  

   offset: The zero-based byte offset in buffer at which to begin storing the data read from the current 

    stream.

  

   count: The maximum number of bytes to be read from the current stream.

   Returns: The total number of bytes read into the buffer. This can be less than the number of bytes 

    requested if that many bytes are not currently available,or zero (0) if the end of the stream 

    has been reached.
  """
  pass
 def ReadAsync(self,buffer,offset,count,cancellationToken=None):
  """
  ReadAsync(self: Stream,buffer: Array[Byte],offset: int,count: int,cancellationToken: CancellationToken) -> Task[int]

  ReadAsync(self: Stream,buffer: Array[Byte],offset: int,count: int) -> Task[int]
  """
  pass
 def ReadByte(self):
  """
  ReadByte(self: Stream) -> int

  

   Reads a byte from the stream and advances the position within the stream by one byte,or returns 

    -1 if at the end of the stream.

  

   Returns: The unsigned byte cast to an Int32,or -1 if at the end of the stream.
  """
  pass
 def Seek(self,offset,origin):
  """
  Seek(self: Stream,offset: Int64,origin: SeekOrigin) -> Int64

  

   When overridden in a derived class,sets the position within the current stream.

  

   offset: A byte offset relative to the origin parameter.

   origin: A value of type System.IO.SeekOrigin indicating the reference point used to obtain the new 

    position.

  

   Returns: The new position within the current stream.
  """
  pass
 def SetLength(self,value):
  """
  SetLength(self: Stream,value: Int64)

   When overridden in a derived class,sets the length of the current stream.

  

   value: The desired length of the current stream in bytes.
  """
  pass
 @staticmethod
 def Synchronized(stream):
  """
  Synchronized(stream: Stream) -> Stream

  

   Creates a thread-safe (synchronized) wrapper around the specified System.IO.Stream object.

  

   stream: The System.IO.Stream object to synchronize.

   Returns: A thread-safe System.IO.Stream object.
  """
  pass
 def Write(self,buffer,offset,count):
  """
  Write(self: Stream,buffer: Array[Byte],offset: int,count: int)

   When overridden in a derived class,writes a sequence of bytes to the current stream and 

    advances the current position within this stream by the number of bytes written.

  

  

   buffer: An array of bytes. This method copies count bytes from buffer to the current stream.

   offset: The zero-based byte offset in buffer at which to begin copying bytes to the current stream.

   count: The number of bytes to be written to the current stream.
  """
  pass
 def WriteAsync(self,buffer,offset,count,cancellationToken=None):
  """
  WriteAsync(self: Stream,buffer: Array[Byte],offset: int,count: int,cancellationToken: CancellationToken) -> Task

  WriteAsync(self: Stream,buffer: Array[Byte],offset: int,count: int) -> Task
  """
  pass
 def WriteByte(self,value):
  """
  WriteByte(self: Stream,value: Byte)

   Writes a byte to the current position in the stream and advances the position within the stream 

    by one byte.

  

  

   value: The byte to write to the stream.
  """
  pass
 def __enter__(self,*args):
  """
  __enter__(self: IDisposable) -> object

  

   Provides the implementation of __enter__ for objects which implement IDisposable.
  """
  pass
 def __exit__(self,*args):
  """
  __exit__(self: IDisposable,exc_type: object,exc_value: object,exc_back: object)

   Provides the implementation of __exit__ for objects which implement IDisposable.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __reduce_ex__(self,*args):
  pass
 CanRead=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """When overridden in a derived class,gets a value indicating whether the current stream supports reading.



Get: CanRead(self: Stream) -> bool



"""

 CanSeek=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """When overridden in a derived class,gets a value indicating whether the current stream supports seeking.



Get: CanSeek(self: Stream) -> bool



"""

 CanTimeout=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that determines whether the current stream can time out.



Get: CanTimeout(self: Stream) -> bool



"""

 CanWrite=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """When overridden in a derived class,gets a value indicating whether the current stream supports writing.



Get: CanWrite(self: Stream) -> bool



"""

 Length=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """When overridden in a derived class,gets the length in bytes of the stream.



Get: Length(self: Stream) -> Int64



"""

 Position=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """When overridden in a derived class,gets or sets the position within the current stream.



Get: Position(self: Stream) -> Int64



Set: Position(self: Stream)=value

"""

 ReadTimeout=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value,in miliseconds,that determines how long the stream will attempt to read before timing out.



Get: ReadTimeout(self: Stream) -> int



Set: ReadTimeout(self: Stream)=value

"""

 WriteTimeout=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value,in miliseconds,that determines how long the stream will attempt to write before timing out.



Get: WriteTimeout(self: Stream) -> int



Set: WriteTimeout(self: Stream)=value

"""


 Null=None




"""Markdown Treeprocessors."""
from __future__ import absolute_import
from __future__ import unicode_literals

from . import inlinepatterns
from . import odict
from . import util


def build_treeprocessors(md_instance, **kwargs):
    """Build the default treeprocessors for Markdown."""
    treeprocessors = odict.OrderedDict()
    treeprocessors["inline"] = InlineProcessor(md_instance)
    treeprocessors["prettify"] = PrettifyTreeprocessor(md_instance)
    return treeprocessors


def isString(s):
    """Check if it's string."""
    if not isinstance(s, util.AtomicString):
        return isinstance(s, util.string_type)
    return False


class Treeprocessor(util.Processor):
    """Treeprocessors are run on the ElementTree object before serialization.

    Each Treeprocessor implements a "run" method that takes a pointer to an
    ElementTree, modifies it as necessary and returns an ElementTree
    object.

    Treeprocessors must extend markdown.Treeprocessor.

    """
    def run(self, root):
        """Main treeprocessor method.

        Subclasses of Treeprocessor should implement a `run` method, which
        takes a root ElementTree. This method can return another ElementTree
        object, and the existing root ElementTree will be replaced, or it can
        modify the current tree and return None.
        """
        pass  # pragma: no cover


class InlineProcessor(Treeprocessor):
    """A Treeprocessor that traverses a tree, applying inline patterns."""

    def __init__(self, md):
        self.__placeholder_prefix = util.INLINE_PLACEHOLDER_PREFIX
        self.__placeholder_suffix = util.ETX
        self.__placeholder_length = 4 + len(self.__placeholder_prefix) \
                                      + len(self.__placeholder_suffix)
        self.__placeholder_re = util.INLINE_PLACEHOLDER_RE
        self.markdown = md
        self.inlinePatterns = md.inlinePatterns

    def __makePlaceholder(self, type):
        """Generate a placeholder."""
        id = "%04d" % len(self.stashed_nodes)
        hash = util.INLINE_PLACEHOLDER % id
        return hash, id

    def __findPlaceholder(self, data, index):
        """Extract id from data string, start from index.

        Args:
            data (str): text
            index (int): index from which we start search

        Returns:
            (str, int): placeholder id and index after the found placeholder.
        """
        m = self.__placeholder_re.search(data, index)
        if m:
            return m.group(1), m.end()
        else:
            return None, index + 1

    def __stashNode(self, node, type):
        """Add node to stash."""
        placeholder, id = self.__makePlaceholder(type)
        self.stashed_nodes[id] = node
        return placeholder

    def __handleInline(self, data, patternIndex=0):
        """Replace placeholders of texts with inline patterns.

        Args:
            data (str): A line of Markdown text
            patternIndex (int): The index of the inlinePattern to start with

        Returns:
            (str): Text with placeholders.
        """
        if not isinstance(data, util.AtomicString):
            startIndex = 0
            while patternIndex < len(self.inlinePatterns):
                data, matched, startIndex = self.__applyPattern(
                    self.inlinePatterns.value_for_index(patternIndex),
                    data, patternIndex, startIndex)
                if not matched:
                    patternIndex += 1
        return data

    def __processElementText(self, node, subnode, isText=True):
        """Process text placeholders of stashed nodes.

        Args:
            node (Element): parent node
            subnode (Element): processing node
            isText (bool): True - it's text, False - it's tail
        """
        if isText:
            text = subnode.text
            subnode.text = None
        else:
            text = subnode.tail
            subnode.tail = None

        childResult = self.__processPlaceholders(text, subnode, isText)

        if not isText and node is not subnode:
            pos = list(node).index(subnode) + 1
        else:
            pos = 0

        childResult.reverse()
        for newChild in childResult:
            node.insert(pos, newChild)

    def __processPlaceholders(self, data, parent, isText=True):
        """Process string with placeholders and generate ElementTree tree.

        Args:
            data (str): text with placeholders instead of ElementTree elements.
            parent (Element): Element which contains processing inline data
            isText (bool): True - it's text, False - it's tail

        Returns:
            (list[ElementTree]): elements with applied inline patterns.
        """
        def linkText(text):
            if text:
                if result:
                    if result[-1].tail:
                        result[-1].tail += text
                    else:
                        result[-1].tail = text
                elif not isText:
                    if parent.tail:
                        parent.tail += text
                    else:
                        parent.tail = text
                else:
                    if parent.text:
                        parent.text += text
                    else:
                        parent.text = text
        result = []
        strartIndex = 0
        while data:
            index = data.find(self.__placeholder_prefix, strartIndex)
            if index != -1:
                id, phEndIndex = self.__findPlaceholder(data, index)

                if id in self.stashed_nodes:
                    node = self.stashed_nodes.get(id)

                    if index > 0:
                        text = data[strartIndex:index]
                        linkText(text)

                    if not isString(node):  # it's Element
                        for child in [node] + list(node):
                            if child.tail:
                                if child.tail.strip():
                                    self.__processElementText(
                                        node, child, False
                                    )
                            if child.text:
                                if child.text.strip():
                                    self.__processElementText(child, child)
                    else:  # it's just a string
                        linkText(node)
                        strartIndex = phEndIndex
                        continue

                    strartIndex = phEndIndex
                    result.append(node)

                else:  # wrong placeholder
                    end = index + len(self.__placeholder_prefix)
                    linkText(data[strartIndex:end])
                    strartIndex = end
            else:
                text = data[strartIndex:]
                if isinstance(data, util.AtomicString):
                    # We don't want to loose the AtomicString
                    text = util.AtomicString(text)
                linkText(text)
                data = ""

        return result

    def __applyPattern(self, pattern, data, patternIndex, startIndex=0):
        """Add the pattern to stashed_nodes if the line fits the pattern.

        Check if the line fits the pattern, create the necessary
        elements, add it to stashed_nodes.

        Args:
            data (str): the text to be processed
            pattern (str): the pattern to be checked
            patternIndex (int): index of current pattern
            startIndex (int): string index, from which we start searching

        Returns:
            (str): text with placeholders instead of ElementTree elements.
        """
        match = pattern.getCompiledRegExp().match(data[startIndex:])
        leftData = data[:startIndex]

        if not match:
            return data, False, 0

        node = pattern.handleMatch(match)

        if node is None:
            return data, True, len(leftData)+match.span(len(match.groups()))[0]

        if not isString(node):
            if not isinstance(node.text, util.AtomicString):
                # We need to process current node too
                for child in [node] + list(node):
                    if not isString(node):
                        if child.text:
                            child.text = self.__handleInline(
                                child.text, patternIndex + 1
                            )
                        if child.tail:
                            child.tail = self.__handleInline(
                                child.tail, patternIndex
                            )

        placeholder = self.__stashNode(node, pattern.type())

        return "%s%s%s%s" % (leftData,
                             match.group(1),
                             placeholder, match.groups()[-1]), True, 0

    def run(self, tree):
        """Apply inline patterns to a parsed Markdown tree.

        Iterate over ElementTree, find elements with inline tag, apply inline
        patterns and append newly created Elements to tree.  If you don't
        want to process your data with inline paterns, instead of normal
        string, use subclass AtomicString:

            node.text = markdown.AtomicString("This will not be processed.")

        Args:
            tree (ElementTree): Markdown tree.

        Returns:
            (ElementTree): Tree with applied inline patterns.
        """
        self.stashed_nodes = {}

        stack = [tree]

        while stack:
            currElement = stack.pop()
            insertQueue = []
            for child in currElement:
                if child.text and not isinstance(
                    child.text, util.AtomicString
                ):
                    text = child.text
                    child.text = None
                    lst = self.__processPlaceholders(
                        self.__handleInline(text), child
                    )
                    stack += lst
                    insertQueue.append((child, lst))
                if child.tail:
                    tail = self.__handleInline(child.tail)
                    dumby = util.etree.Element('d')
                    child.tail = None
                    tailResult = self.__processPlaceholders(tail, dumby, False)
                    if dumby.tail:
                        child.tail = dumby.tail
                    pos = list(currElement).index(child) + 1
                    tailResult.reverse()
                    for newChild in tailResult:
                        currElement.insert(pos, newChild)
                if len(child):
                    stack.append(child)

            for element, lst in insertQueue:
                if self.markdown.enable_attributes:
                    if element.text and isString(element.text):
                        element.text = inlinepatterns.handleAttributes(
                            element.text, element
                        )
                i = 0
                for newChild in lst:
                    if self.markdown.enable_attributes:
                        # Processing attributes
                        if newChild.tail and isString(newChild.tail):
                            newChild.tail = inlinepatterns.handleAttributes(
                                newChild.tail, element
                            )
                        if newChild.text and isString(newChild.text):
                            newChild.text = inlinepatterns.handleAttributes(
                                newChild.text, newChild
                            )
                    element.insert(i, newChild)
                    i += 1
        return tree


class PrettifyTreeprocessor(Treeprocessor):
    """Add linebreaks to the html document."""

    def _prettifyETree(self, elem):
        """Recursively add linebreaks to ElementTree children."""
        i = "\n"
        if util.isBlockLevel(elem.tag) and elem.tag not in ['code', 'pre']:
            if (not elem.text or not elem.text.strip()) \
                    and len(elem) and util.isBlockLevel(elem[0].tag):
                elem.text = i
            for e in elem:
                if util.isBlockLevel(e.tag):
                    self._prettifyETree(e)
            if not elem.tail or not elem.tail.strip():
                elem.tail = i
        if not elem.tail or not elem.tail.strip():
            elem.tail = i

    def run(self, root):
        """Add linebreaks to ElementTree root object."""
        self._prettifyETree(root)
        # Do <br />'s seperately as they are often in the middle of
        # inline content and missed by _prettifyETree.
        brs = root.iter('br')
        for br in brs:
            if not br.tail or not br.tail.strip():
                br.tail = '\n'
            else:
                br.tail = '\n%s' % br.tail
        # Clean up extra empty lines at end of code blocks.
        pres = root.iter('pre')
        for pre in pres:
            if len(pre) and pre[0].tag == 'code':
                pre[0].text = util.AtomicString(pre[0].text.rstrip() + '\n')



class ArgumentOutOfRangeException(ArgumentException,ISerializable,_Exception):
 """
 The exception that is thrown when the value of an argument is outside the allowable range of values as defined by the invoked method.

 

 ArgumentOutOfRangeException()

 ArgumentOutOfRangeException(paramName: str)

 ArgumentOutOfRangeException(paramName: str,message: str)

 ArgumentOutOfRangeException(message: str,innerException: Exception)

 ArgumentOutOfRangeException(paramName: str,actualValue: object,message: str)
 """
 def add_SerializeObjectState(self,*args):
  """ add_SerializeObjectState(self: Exception,value: EventHandler[SafeSerializationEventArgs]) """
  pass
 def GetObjectData(self,info,context):
  """
  GetObjectData(self: ArgumentOutOfRangeException,info: SerializationInfo,context: StreamingContext)

   Sets the System.Runtime.Serialization.SerializationInfo object with the invalid argument value 

    and additional exception information.

  

  

   info: The object that holds the serialized object data.

   context: An object that describes the source or destination of the serialized data.
  """
  pass
 def remove_SerializeObjectState(self,*args):
  """ remove_SerializeObjectState(self: Exception,value: EventHandler[SafeSerializationEventArgs]) """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,*__args):
  """
  __new__(cls: type)

  __new__(cls: type,paramName: str)

  __new__(cls: type,paramName: str,message: str)

  __new__(cls: type,message: str,innerException: Exception)

  __new__(cls: type,paramName: str,actualValue: object,message: str)

  __new__(cls: type,info: SerializationInfo,context: StreamingContext)
  """
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 ActualValue=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the argument value that causes this exception.



Get: ActualValue(self: ArgumentOutOfRangeException) -> object



"""

 Message=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the error message and the string representation of the invalid argument value,or only the error message if the argument value is null.



Get: Message(self: ArgumentOutOfRangeException) -> str



"""


